﻿;;; <?xml version="1.0" encoding="UTF-8"?>

;;; Terregov Ontology V9 SOL format (30/9/5)

;;; Structure
;;; =========
;;; 1. Geographical areas, Territories and Administrative zones
;;; 1.1 Geographic areas and territories
;;; 1.2 Administrative zones
;;; 2. Authorities
;;; 2.1 Elected bodies
;;; 2.2 Non elected bodies, local or state authorities
;;; 3. Social services
;;; 3.1 Employment
;;; 3.2 Health care
;;; 3.3 Service domains, housing
;;; 3.4 Assistance
;;; 3.4.1 Financial assistance
;;; 3.4.2 Program
;;; 3.4.3 Payment, revenue, salary
;;; 3.5 Education
;;; 3.6 Working domains, professional activities
;;; 4. Documents
;;; 4.1 Legal documents
;;; 4.2 Medical documents
;;; 4.3 Evidence Place Holder
;;; 5. Facilities, buildings
;;; 6. Organizations
;;; 7. People
;;; 8. Time
;;; 9. Information Technology
;;; 10. Miscellaneous

(setq *language* :all
      moss::*verbose* t)

(defontology 
  (:version 1.0)
  (:doctype "owl" "http://www.w3.org/2002/07/owl#"
            "terregov" "http://www.terregov.eupn.net/ontology/2005/10/terregov.owl/terregov#"
            "xsd"       "http://www.w3.org/2001/XMLSchema#" )
  (:xmlns "rdf" "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
          "rdfs" "http://www.w3.org/2000/01/rdf-schema#"
          "owl" "http://www.w3.org/2002/07/owl#"
          "xsd" "&xsd;"
          "self" "&terregov;"
          "terregov" "&terregov;")
  (:base "http://www.terregov.eupn.net/ontology/2005/10/terregov.owl/terregov")
  (:imports "http://www.w3.org/2000/01/rdf-schema"
            "http://www.w3.org/1999/02/22-rdf-syntax-ns"
            "http://www.w3.org/2002/07/owl")
  (:package :terregov)
  (:language :all)
  )

#|
<!ENTITY terregov "http://.../terregov#">
xmlns          ="&terregov;"
xmlns:terregov  ="&terregov;"
|#

(defchapter (:en "Concepts" :fr "Concepts" :pl "Koncept") (:no-number))

;;;--------------------- 1. Geographical areas, Territories and Administrative zones

(defchapter (:en "Geographical Areas, Territories and Administrative Zones"
                 :fr "Zones Géographique, Territoires et Zones Administratives"
                 :it "Aree Geografiche, Territori"
                 :pl "Obszary geograficzne, terytoria i strefy administracyjne"))
;;  (:no-number) (:number 3))

;;;-------------------------------------------- 1.1 geographic areas and territories

(defsection (:en "Geographic areas and territories"
                 :fr "Zones Géographique et Territoires"
                 :it  "Aree Geografiche"
                 :pl "obszary geograficzne i terytoria"))

(defconcept 
  (:name :en "Geographical area" 
         :fr "Zone géographique" 
         :it "Area Geografica" 
         :pl "obszar geograficzny" )
  (:doc :en "A Geographical Area is a part of a territory."
        :fr "Une zone géographique est une partie d'un territoire."
        :pl "Obszar geograficzny to część terytorium."))


(defconcept 
  (:name :en "Territory" :fr "Territoire" :it "territorio" :pl "terytorium")
  (:att (:en "name" :fr "nom" :pl "nazwa") (:type :name)(:unique))
  (:doc :en "A territory is a part of a geographical entity under the rule of nation ~
             or a part of it."
        :fr "Un territoire est une étendue de terre qu'offre un État, une province, ~
             une ville, une juridiction, etc."
        :pl "Terytorium to część jednostki geograficznej znajdująca się pod ~
             rządami jednego narodu lub jego części."))

;;;-------------------------------------------------------- 1.2 Administrative zones
(defsection (:en "Administrative Zones"
                 :fr "Zones Administratives"
                 :pl "Administracyjne Strefy "))

(defconcept 
  (:name :en "City; town" :fr "ville; cité" :it "Citta"  :pl "miasto")
  (:is-a "territory")
  (:doc :en "A large and densely populated urban area; may include several ~
             independent administrative districts."
        :fr "Une ville est une zone urbaine peuplée de façon dense pouvant inclure ~
             des districts administratifs indépendants."
        :pl "Duży, gęsto zaludniony obszar miejski; może zawierać kilka ~
             niezależnych administracyjnie dzielnic."))


(defconcept 
  (:name :en "Country; land" :fr "pays" :it "Stato" :pl "Kraj, państwo")
  (:is-a "territory")
  (:doc :en "a Country or a state is an administrative entity."
        :fr "Territoire qui appartient à une nation, qui est administré par un ~
             gouvernement et dont les frontières terrestres et maritimes ont ~
             clairement été établies. "
        :pl "Kraj lub państwo to jednostka administracyjna."))

(defconcept 
  (:name :en "County" :fr "comté" )
  (:is-a "Territory")
  (:doc :en "A county is a major administrative, judicial and political district ~
             into which Great Britain is divided."
        :fr "Un comté est une circonscription administrative britannique."))

(defconcept 
  (:name :en "French department" :fr "département" :it "Provincia")
  (:is-a "Territory")
  (:att (:name :en "code" :fr "code" :pl "kod")(:type :string))
  (:rel (:name :fr "région" :en "region" :pl "region")(:to "region")(:unique))
  (:rel (:en "prefecture" :fr "préfecture") (:to "prefecture") (:unique) )
  ;(:rel (:name :en "sub prefecture" :fr "sous-préfecture")(:to "sub-prefecture"))
  (:doc :en "A French department is a French administrative entity."
        :fr "Un département est une des Principales divisions administratives du ~
             territoire français."))


(defconcept 
  (:name :en "Region" :fr "région" :it "Regione" :pl "Region")
  (:is-a "territory")
  (:rel (:name :en "french department" :fr "département")(:to "département"))
  (:rel (:name :en "country" :fr "pays" :pl "kraj")(:to "country"))
  (:doc :en "A region is a large administrative area."
        :fr "Unité territoriale administrative à fonctions économiques, sociales, ~
             culturelles, groupant généralement plusieurs départements."
        :pl "Region to duży obszar administracyjny."))

;;;------------------------------------------------------------------ 2. Authorities

(defchapter (:en "Authorities"
                 :fr "Autorités"
                 :pl "Jednostka nadzorująca" ))

;;;-------------------------------------------------------------- 2.1 Elected Bodies

(defsection (:en "Elected Bodies" :fr "Autorités élues" :it "Rappresentanti Eletti" 
                 :pl "Ciało wybieralne "))


(defconcept 
  (:name :en "Authority" :fr "Autorité" :it "Ente Autoritario" :pl "Władza" )
  (:att (:en "name" :fr "nom" :pl "nazwa") (:type :name) (:unique))
  (:rel (:en "management responsibility" :pl "odpowiedzialność zarządcza") 
        (:to "territory") (:unique))
  (:doc :en "An Authority is a legal body operating at the level of Europe, of a ~
             country, a department, a city."
        :fr "L'autorité comprend les magistrats, les hauts fonctionnaires chargés ~
             d'une partie quelconque de l'administration publique."
        :pl "Władza to powołane zgodnie z prawem ciało funkcjonujące na poziomie ~
             Europy, kraju, miasta czy wydziału."))

(defconcept 
  (:name :en "Local Authority" :fr "autorité locale" :it "Ente Autoritario Locale" 
         :pl "Władze lokalne")
  (:is-a "Authority")
  ;(:disjoint "State Authority")
  (:doc :en "A local authority is not a state authority."
        :fr "Une autorité locale a un pouvoir local. Elle est opposée à une ~
             autorité nationale. Ensemble des décideurs élus ou nommés qui ~
             représentent une collectivité locale précise."
        :pl "Władze lokalne nie są władzami państwa."))

;;;------------------------------ 2.2 Non elected bodies, local or state authorities

(defsection (:en "Non elected bodies, local or state authorities"
                 :fr "Autorités non élues, Autorités locales ou nationales"))

(defconcept 
  (:name :en "Ministry" :fr "ministère" :it "Ministero" :pl "Ministerstwo")
  ;(:is-a "State Authority")
  (:att (:en "name" :fr "nom" :pl "nazwa") (:type :name) (:unique))
  (:doc :en "A governmental department presided over by a minister."
        :fr "MINISTÈRE signifie particulièrement Fonction des ministres chargés du ~
             soin des affaires publiques et dont l'ensemble constitue le gouvernement. ~
             Il désigne spécialement le Département qu'ils administrent."
        :pl "Organ władzy państwowej, któremu przewodniczy minister."))

(defconcept 
  (:name :en "Prefecture" :fr "Préfecture" :it "Prefettura")
  (:is-a "Local Authority")
  (:att (:en "name" :fr "nom") (:type :name)(:unique))
  (:doc :en "A Prefecture is the local authority that represents the state at the ~
             department level."
        :fr "Préfecture désigne, dans l'organisation administrative actuelle de ~
             la France, la Charge de l'administrateur de département appelé Préfet."))


;;;=============================== individuals ======================================
(defchapter (:en "Individuals" :fr "Individus" :it "Individui") (:no-number))

;;;------------------------ 1. geographic areas, Territories and Administrative zones
(defchapter (:en "Geographical Areas, Territories and Administrative Zones"
                 :fr "Zones Géographique, Territoires et Zones Administratives" 
                 :pl "Obszary geograficzne, terytoria i strefy administracyjne") 
  (:number 1))



;;;--------------------------------------------- 1.1 geographic areas and territories
(defsection (:en "Geographic areas and territories"
                 :fr "Zones Géographique et Territoires"
                 :pl "Obszary geograficzne i terytoria "))

;;;----- COUNTRY

(defindividual "country"
  (:name :en "France" :fr "France" :it "Francia" :pl "Francja")
  ("name" (:en "France" :fr "France" :it "Francia" :pl "Francja")))

(defindividual "country"
  (:name :en "Italy" :fr "Italie" :it "Italia" :pl "Włochy")
  ("name" (:en "Italy" :fr "Italie" :it "Italia" :pl "Włochy")))

(defindividual "country"
  (:name :en "poland" :fr "Pologne" :it "Polonia" :pl "Polska")
  ("name" (:en "poland" :fr "Pologne" :it "Polonia" :pl "Polska")))

(defindividual "country"
  (:name :en "united kingdom" :fr "Royaume Uni" :it "Inghilterra" :pl "Anglia")
  ("name" (:en "united kingdom" :fr "Royaume Uni" :it "Inghilterra" :pl "Anglia")))

;;;--------------------------------------------------------- 1.2 Administrative zones

(defsection (:en "Administrative Zones"
                 :fr "Zones Administratives"
                 :pl "strefy administracyjne"))

;;;----- DEPARTMENTS

(defindividual "département"
  (:name :fr "dordogne")
  ("name" "Dordogne")
  ("region" "aquitaine")
  ("code" "24"))

(defindividual "département"
  (:name :fr "gironde")
  ("name" "Gironde")
  ("region" "aquitaine")
  ("préfecture" "bordeaux")
  ("code" "33"))


;;;----- PREFECTURE

(defindividual "préfecture"
  (:name :fr "bordeaux")
  ("name" "bordeaux"))

;;;----- REGION

(defindividual "région"
  (:name :fr "aquitaine")
  ("name" "Aquitaine")
  ("country" "france"))

(defindividual "region"
  (:name :it "regione veneto")
  ("name" "Regione Veneto")
  ("country" "italy"))

;;;==================================================================================

:EOF