﻿/*
 * Project: Ontocodesign
 * Set of functions used to display concepts and relationships using a radial graph
 *
 * History
 * 30/09/2014 MFK Creation
 * 03/10/2014 i18n
 */

// i18n keyset
var keys = {};

// init i18n keyset
initKeys(keys);

// autocomplete function
$(document).ready(function() {
  // get language
  var language = getLanguage(window.location.href);
  var concepts = [];
  document.getElementById("conceptsearch").placeholder = getText("loading");
  var url = "../concept" + (language == null ? "" : "?" + language);
  // get all concept names
  $.getJSON(url,function(result){
    document.getElementById("conceptsearch").placeholder = getText("placeholder");
    $.each(result, function(i, field){
      concepts.push({ value: field.label[0] });
    });
    $('#conceptsearch').autocomplete({
      lookup: concepts,
      onSelect: function (suggestion,data) {
        if(suggestion == null) {
          return false;
        }
        $('#msg').append("<p>" + getText("loading") + "</p>");
        url = "../concept?label=" + suggestion.value.split(",")[0] +
                    (language == null ? "" : "&" + language);
        $.getJSON(url ,
          function(result) {
            if(result != null) {
              $('#msg').empty();
              $('#outputbox').empty();
              st = renderTree(result,document.getElementById('outputbox'),language);
              $('#outputbox').append("<div class=\"navbuttons\"><table>" +
                    "<tr><td><a href=\"javascript:zoomIn()\"><img src=\"images/plus.png\"></a></td><td></td><td><a href=\"javascript:zoomOut()\"><img src=\"images/minus.png\"></a></td></tr>" +
                    "<tr><td></td><td><a href=\"javascript:up()\"><img src=\"images/up.png\"></a></td><td></td></tr>" +
                    "<tr><td><a href=\"javascript:left()\"><img src=\"images/left.png\"></a></td><td></td><td><a href=\"javascript:right()\"><img src=\"images/right.png\"></a></td></tr>" +
                    "<tr><td></td><td><a href=\"javascript:down()\"><img src=\"images/down.png\"></a></td><td></td></tr>" +
                        "</table></div>");
              } else {
              alert("No results found");
            }
          });
      }
    });  
  });
});

function zoomIn() {
  var val = st.controller.Navigation.zooming/1000; 
  var ans = 1 + (10 * val);
  st.canvas.scale(ans, ans);
}

function up() {
  st.canvas.translate(0,10);
}

function left() {
  st.canvas.translate(10,0);
}

function right() {
  st.canvas.translate(-10,0);
}

function down() {
  st.canvas.translate(0,-10);
}

function zoomOut() {
  var val = st.controller.Navigation.zooming/1000; 
  var ans = 1 - (10 * val);
  st.canvas.scale(ans, ans);
}

/*
 * Initialize the i18n keyset
 * arguments: 
 *   keys: the keys associative array
 */
function initKeys(keys) {
  keys["title_FR"] = "Afficher comme un arbre";
  keys["title_EN"] = "Display as a Tree";
  keys["msg_FR"] = "Il suffit de commencer à taper et les résultats seront affichés ici";
  keys["msg_EN"] = "Just start typing and results will be displayed here";
  keys["back_EN"] = "Home";
  keys["back_FR"] = "Retour vers la page d'accueil";
  keys["placeholder_FR"] = "Tapez le concept";
  keys["placeholder_EN"] = "Enter concept";
  keys["loading_EN"] = "Loading ... ";
  keys["loading_FR"] = "Chargement en cours ...";
}

/*
 * Get language parameter (complete)
 * arguments: 
 *   url: the given url
 * return:
 *   a string containing variable and content in the format (e.g. "language=EN")
 */
function getLanguage(url) {
  var re = "language=[A-Z][A-Z]";
  var language = url.match(re);
  if(language == null) {
    return null;
  }  
  return language[0];
}

/*
 * Replace the object content with the corresponding i18n value
 * arguments: 
 *   key: the i18n label
 *   id: the html object id
 */
function setText(key,id) {
  key = key+"_"+getLanguageFragment();
  document.getElementById(id).innerHTML = keys[key];
}

/*
 * Get the i18n value given a label
 * arguments: 
 *   key: the i18n label
 * returns:
 *   the corresponding i18n value
 */
function getText(key) {
  key = key+"_"+getLanguageFragment();
  return keys[key];
}

/*
 * Get the language fragment of the current url
 * returns:
 *   the language value (e.g.: FR, EN)
 */
function getLanguageFragment() {
  var ln = getLanguage(window.location.href);
  if(ln == null) {
    return "EN";
  } else {
    return ln.substring(ln.indexOf("=")+1);
  }
}

/* EOF */

