﻿;;;-*- Mode: Lisp; Package: "NEWS" -*-
;;;==============================================================================
;;;10/12/07
;;;                               AGENT NEWS
;;;
;;;==============================================================================

;;; we call our agent NEWS to avoid conflicts sith the NEWS symbol used in
;;; the ontology

#|
2009
 0721 Changing for the environment 7.10
 0722 Adding usage of ontology for get-news-by-section
 0728 Adding get-all-sections skill (jpb)
2010
 0214 adding ping and hello 
 1122 adapting to OMAS v8.0.8
 1207 testing persistency
 |#

(defpackage :NEWS (:use :cl #+MCL :ccl :moss :omas))
(in-package :NEWS)

;;;===============================================================================
;;;                         debugging macros 
;;;===============================================================================

(defUn ipa () (in-package :albert))
(defUn ips () (in-package :stevens))
(defUn ipl () (in-package :alfredo))
(defUn ipcg () (in-package :cg-user))
(defUn ipcl () (in-package :cl-user))
(defUn ipm () (in-package :moss))

;;;===============================================================================
;;; 
;;;                         creating agent
;;;
;;;===============================================================================

(omas::defagent :NEWS :redefine t :persistency t) ; JPB0902

;;;===============================================================================
;;; 
;;;                         service macros and functions 
;;;
;;;===============================================================================


;;; ==================================== globals =================================
;;; Globals  can be used within the agent space to simplify programming, although
;;; it is better to use the agent memory.

(defParameter *news-words* 
  '("news" "new" "section" "about"
    "nouvelle" "nouveau" "rubrique" "page" "section" "sur" "nouvelles"
    "noticia" "nueva" "nuevo" "sección" "noticias" "nuevas" "sobre"))

(defParameter *empty-words* 
  '("the" "a" "an" "of" "from" "to" "those" "these" "that" "s" "?"
    "le" "la" "les" "de" "du" "des" "à" "aux" "ce" "ces" "d" "l"
    "la" "el" "los" "las" "de" "del" "a" "al" "un" "una" "aquellos" "estos" "eso" 
    "esto" "son" "cuales"))

;;;===============================================================================
;;; 
;;;                                    skills 
;;;
;;;===============================================================================


;;;========================================================================= skill
;;;                              GET LAST NEWS
;;;===============================================================================

(defUn get-last-news ()
  "returns the last 3 entries in the news using the class counter"
  (let (concept-id counter-id next-value result)
    ;; get news concept
    (setq concept-id 
          (car (access '("moss-concept" ("moss-concept-name" :is "news")))))
    (unless concept-id (return-from get-last-news))
    
    ;; counter
    (setq counter-id (car (send concept-id '=get "moss-counter")))
    (unless counter-id (return-from get-last-news))
    
    ;; counter value for the next instance
    (setq next-value (car (send counter-id '=get "moss-value")))
    (unless (numberp next-value) (return-from get-last-news))
    
    ;; use the counter value to access last instances
    (send concept-id '=get-instances 
          :min (max 1 (- next-value 3)) :max next-value)))
    
;;;========================================================================= skill

(defskill :GET-LAST-NEWS :news
  :static-fcn static-GET-LAST-NEWS
  )

(defUn static-get-last-news (agent message args)
  "returns the last 3 entries in the news using the class counter"
  (declare (ignore message))
  (let ((*language* (or (cdr (assoc :language args)) *language*))
        (pattern (cdr (assoc :pattern args)))
        instances result)
    (setq instances (get-last-news))
    (setq result
          (cond
           ((and instances pattern)
            (reduce #'append (fill-pattern instances pattern)))
           (instances
            (reduce #'append (broadcast instances '=summary)))
           (t
            :failure)))
    (static-exit agent result)))


;;;========================================================================= skill
;;;                            GET NEWS BY SECTION
;;;===============================================================================

;;; Sections are:
;;;   - Culture
;;;   - Economy
;;;   - International
;;;   - OMAS-MOSS
;;;   - Miscellaneous
;;;   - Society
;;;   - Sports
;;;   - TWA
;;;   - Weather

;;;========================================================================= skill

(defskill :GET-NEWS-BY-SECTION :news
  :static-fcn static-GET-NEWS-BY-SECTION
  )	  

(defUn static-get-news-by-section (agent message args)
  (declare (ignore message))
  (let ((data (cdr (assoc :data args)))
        (*language* (or (cdr (assoc :language args)) *language*))
        (pattern (cdr (assoc :pattern args)))
        input instances result)
    
    ;; clean empty words and words like news, section, page, etc
    (setq input (moss::%clean-word-list data *empty-words* *news-words*))    
    ;; If there is nothing but empty words, return failure
    (unless input
      (static-exit agent "failure")
      (return-from static-get-news-by-section nil))
    
    ;; locate objects
    (setq instances (locate-objects input "news"))
    
    ;; if nothing try to get last news
    (if (and (null instances) (locate-objects input "section"))
      (setq result 
            (list "Je n'ai rien sur le sujet, mais voici les dernières entrées...")))

    (unless instances (setq instances (get-last-news)))

    (setq result
          (if instances
            (append result (reduce #'append (fill-pattern instances pattern)))
            :failure))
    (static-exit agent result)))
    
#|
? (static-get-news-by-section SA_NEWS nil
      '((:DATA "Nouvelles" "de" "la" "météo" "?")(:LANGUAGE :FR)))
("2010-11-22 18:26:32 (jpb) : météo
    La température est en sérieuse baisse"
 "2010-11-22 18:29:53 (mr) : météo
    L'hiver arrive...")
|#
;;;======================================================================== skill
;;;                              GET-NEWS-SECTIONS
;;;==============================================================================

(defskill :GET-NEWS-SECTIONS :NEWS
  :static-fcn static-get-news-sections)

(defUn static-get-news-sections (agent message &rest args)
  "Return the list of all available sections"
  (declare (ignore message args))
  (let (result)
    (setq result
          (mapcar #'(lambda(xx)(mln::get-canonical-name (car(has-name xx)))) 
		  (access '(section))))
    (static-exit agent result)))

#|
? (static-get-news-sections SA_NEWS nil :dummy)
("Culture" "Économie" "International" "National" "OMAS-MOSS" "Divers" "Société"
 "Sport" "TWA" "Météo")
|#
;;;======================================================================== skill
;;;                              PUBLISH NEWS
;;;==============================================================================

(defUn current-date-string ()
  "Returns current date as a string."
  (multiple-value-bind (sec min hr day mon yr dow dst-p tz)
                       (get-decoded-time)
    (declare (ignore sec min hr dow dst-p tz))
    (format nil "~A-~A-~A" yr mon day)))

(defUn current-hour-string ()
  "Returns current hour as a string."
  (multiple-value-bind (sec min hr day mon yr dow dst-p tz)
                       (get-decoded-time)
    (declare (ignore day mon yr dst-p tz))
    (format nil "~A:~A:~A" hr min sec)))

;;;======================================================================== skill

(defskill :PUBLISH-NEWS :news
  :static-fcn static-PUBLISH-NEWS
  )

(defUn static-publish-news (agent message args)
  "record the data at the end of the current NEWS.tex file."
  (declare (ignore message)) 
  (let (obj-id)
    (format t "~%; publish news /args:~%  ~S" args) 
    ;; create a NEWS object in a transaction
    (if (omas::persistency agent) (omas::agent-start-changes agent))
    (setq obj-id
          (apply #'moss::%make-individual "news"
                 (append (cdr (assoc :data args))
                         `(("date" ,(current-date-string)))
                         `(("heure" ,(current-hour-string))))))
    (if (omas::persistency agent) (omas::agent-commit-changes agent))
    (format t "~%; publish news /obj-id: ~S" obj-id)

    (static-exit agent :success)
    ))

#|
? (setq args `((:data ("sujet" "Omar at UTC")
                      ("contenu" "Omar comes from Guadelajara")
                      ("section" "international"))))

? (static-publish-news SA_NEWS :dummy-arg args)
? $E-NEWS.5
((MOSS::$TYPE (0 $E-NEWS)) (MOSS::$ID (0 $E-NEWS.5))
 ($T-NEWS-TITLE (0 "Omar at UTC"))
 ($T-NEWS-CONTENT (0 "Omar comes from Guadelajara"))
 ($S-NEWS-SECTION (0 $E-SECTION.3)) ($T-NEWS-DATE (0 "2010-11-22"))
 ($T-NEWS-HOUR (0 "17:25:38")))
|#

:EOF