﻿;;;-*- Mode: Lisp; Package: "DOCTOR" -*-
;;;===============================================================================
;;;11/04/23
;;;             AGENT DOCTOR: DIALOG (file DOCTOR-dialog.lisp)
;;;             Copyright Barthes@UTC, 2006-2010
;;;
;;;=============================================================================== 

;;; Note that this file must be in UTF-8 format (like the other agent files)
;;; The top-level dialog works in English, French, Portuguese (Brasilian), or Japanese
;;; For any other language the questions and answers must be translated and added
;;; to the already existing ones.

#|
DOCTOR is a therapist assistant agent. Its has a single main dialog.
 
2011
 0416 creation
 0423 adding new macro for easing out the dialogs
|#

(in-package :DOCTOR)

;; this is used by the Japanese language, can be removed if not needed
(eval-when (:load-toplevel :compile-toplevel :execute)
  (push 'はい moss::*yes-answers*)
  (push 'OK moss::*yes-answers*)
  (push 'いいえ moss::*no-answers*)
  )
  
;(format t "~&loading DOCTOR dialogs, package is: ~S" *package*)

;;;================================================================================
;;;
;;;                         DOCTOR macros.
;;;
;;;================================================================================

; (eval-when (:compile-toplevel :load-toplevel :execute)
; (defmacro exec-if-format (expr)
   ; "if expr is a format expr, execute it, otherwise return expr"
   ; `(if (and (listp ,expr) (eql 'format (car ,expr)))
      ; (eval ,expr)
	  ; ',expr))
	  ; )
(defun exec-if-format (expr)
   "if expr is a format expr, execute it, otherwise return expr"
   (if (and (listp expr) (eql 'format (car expr)))
      (eval expr)
	  expr))

	  
#|
(exec-if-format "Youpee...")
"Youpee..."
(exec-if-format '(format nil "Number...~S" 2))
"Number...2"
(exec-if-format '("Abert" "Joseph"))
'("Abert" "Joseph")
|#
	  
;;; the following macro is provided to avoid writing a bulky :send-message option
;;; whenever we send a message to the patient. To be used in place of defstate

(defmacro defdocstate (label &rest ll)
   (let ((ask-patient (assoc :ask-patient ll))
         (tell-patient (assoc :tell-patient ll))
		 (topics (assoc :select-topics ll)))
     ;(print ask-patient)
    `(defstate ,label
	    ;; if we have a question, then sned message, print question, wait for answer
		;; and print answer, then remove clause and proceed
		,@(if (and ask-patient tell-patient) 
		   (error "can't have ask and tell in the same state yet"))
        ,@(when ask-patient
		   ;; first remove clause from the list
		   (setq ll (remove :ask-patient ll :key #'car))
		   ;; send a message, wait for answer and print answer
	       `((:send-message :to :patient :self *current-agent* :action :ask 
             ,@(if (member :timeout ask-patient)
			    (list :timeout (getf ask-patient :timeout)))
             :args `(((:data . ,(exec-if-format ,(cadr ask-patient)))
                      (:language . :en)
                          )))
			 ;; print before getting the answer
			 (:execute-preconditions
		        (send moss::conversation 
                  '=display-text
				  (exec-if-format ,(if ask-patient 
				                      (cadr ask-patient)
				                      (cadr tell-patient)))
				  :header "DOCTOR> "
				  :color cg::red))
			 ;; print after the answer has been received
			 (:execute	  
				(send moss::conversation 
                  '=display-text
				  (read-fact moss::conversation :raw-input)
				  :header "PATIENT> "
				  :color cg::blue)
				)
			  )
		    )
		;; if we have a comment, then send comment and print it. No answer 	  
        ,@(when tell-patient
		   ;; first remove clause from the list
		   (setq ll (remove :tell-patient ll :key #'car))
		   ;; then produce the clauses
	       `((:send-message :to :patient :self *current-agent* :action :tell
               :type :inform		   
               :args `(((:data . ,(exec-if-format ,(cadr tell-patient)))
                        (:language . :en)
                          )))
		     (:execute-preconditions
		       (send moss::conversation 
                  '=display-text
				  (exec-if-format ,(if ask-patient 
				                      (cadr ask-patient)
				                      (cadr tell-patient)))
				  :header "DOCTOR> "
				  :color cg::red)))
			)
		;; topic selection
		,@(when topics
		    (setq ll (remove :select-topics ll :key #'car))
			;; build the clauses
			;; the post-topics skill posts a list of topics and asks a question
            `((:send-message :to :patient :self :doctor 
                 :action :post-topics
                 :args '(((:data ,(cadr topics) ,(getf topics :question)
                          ))))
              ;; post the text
              (:execute-preconditions
	           (send moss::conversation 
                  '=display-text ,(getf topics :question)
	              :header "DOCTOR> "
		          :color cg::red))
              ;; on selection of a topic, we get the answer somewhere	²
              (:transitions
			   ,@(loop for item in (cadr topics) collect
			       `(:patterns ((,item)) 
				        :sub-dialog 
						  ,(intern 
						     (format nil "_~A-CONVERSATION"
							          (string-upcase item)))))
				(:otherwise :target ,(getf topics :failure))
				)
			  ;; print after the answer has been received
			  (:execute	  
				(send moss::conversation 
                  '=display-text
				  (read-fact moss::conversation :raw-input)
				  :header "PATIENT> "
				  :color cg::blue)
				)			 
			 ))
        ;; finally add the rest of clauses
		,@ll)))
		
#|
(defdocstate 
     _doc-greetings
  (:label "Greetings")
  (:explanation "Doctor is greeting patient.")
  (:transitions
    (:always :target _doc-greetings)
   ))

(defdocstate 
     _doc-greetings
  (:label "Greetings")
  (:explanation "Doctor is greeting patient.")
  (:ask-patient "Hello!")
  (:transitions
    (:always :target _doc-greetings)
   ))
   
(defdocstate
  _doc-show-topics
  (:label "Show topics")
  (:explanation "Doctor is selecting a list of topics to show patient.")
  (:select-topics 
     ("Palace" "Nijo" "Kiyomizu")
     :question
     "Palace, Nijo Castle, Kiyomizu Temple: have you ever been to ~
		   any place listed here? If so, select it."
     :failure _doc-no-visit)
  )
|#
;;;================================================================================
;;;
;;;             Sub-Conversations, Examples are in French...
;;;
;;;================================================================================

;;; Sub-conversations are defined by a conversation header, that points to the
;;; entry state.
;;; There are two output states:
;;;   SUCCESS: the result is stored into the RESULTS slot of the state-context
;;;   FAILURE: no result could be achieved
;;; In case of bad error there is a throw to an :error label
;;; data are obtained from the HAS-DATA slot of the context object

;;;================================================================================
;;;
;;;                      GENERIC TASK CONVERSATION
;;;
;;;================================================================================

;;; this subconversation takes a task model and fills in the required parameters
;;; before shipping the data (for possible optional args) and filled pattern to the 
;;; specialist

;;; probably not very useful

;;;================================================================================
;;;
;;;                       PRINT HELP CONVERSATION
;;;
;;;================================================================================

(defsubdialog 
  _doctor-conversation 
  (:label "Doctor conversation")
  (:explanation "This is the main conversation driven by the Doctor.")
  (:states _doc-entry-state  ; required by defstate
           _doc-get-name
		   _doc-got-nme
		   _doc-greetings
		   _doc-no-visit
		   _doc-show-topics
		   )
  )

;;;------------------------------------------------------------- (DOC) ENTRY-STATE

(defdocstate
  _doc-entry-state
  (:label "doctor dialog entry")
  (:entry-state _doctor-conversation )
  (:explanation "Doctor is starting session.")
  (:ask-patient "Hello!")
  (:transitions
    (:always :target _doc-get-name))
  ) 
  
;;;---------------------------------------------------------------- (DOC) GET-NAME 

(defdocstate
  _doc-get-name
  (:label "Greetings")
  (:explanation "Doctor is asking patient's name.")
  (:ask-patient "What is your name?")
  (:transitions
    (:on-failure :target _doc-greetings)
    (:patterns
	   (((?* ?x) "my" "name" "is" (?* ?y))
	    ((?* ?y)))
		:keep ?y :target _doc-got-name)
    (:otherwise :target _doc-greetings))
  )

;;;---------------------------------------------------------------- (DOC) GOT-NAME 

(defdocstate
  _doc-got-name
  (:label "Greetings with name")
  (:explanation "Doctor is using patient's name.")
  (:tell-patient (format nil "Hi, ~{~A~^ ~}. Nice meeting you.
  Now we are going to talk using topics displayed on the screen."
                    (read-fact moss::conversation '?y)))
  (:no-wait)
  (:transitions
    ;; wait 2s to let the patient read the text
    (:always :exec (sleep 2) :target _doc-show-topics))
  )

;;;--------------------------------------------------------------- (DOC) GREETINGS

(defdocstate 
  _doc-greetings
  (:label "Greetings")
  (:explanation "Doctor is greeting patient.")
  (:tell-patient "Nice to meet you.
Now we are going to talk using topics displayed on the screen.")
  (:no-wait)
  (:transitions
    (:always :exec (sleep 2) :target _doc-show-topics)
   ))
   
;;;----------------------------------------------------------------- (DOC) NO-VISIT

(defdocstate
  _doc-no-visit
  (label "No visit")
  (explanation "Patient did not answer or answered negatively.")
  (:ask-patient "Ah, have you been,...?")
  (:transitions
     (:always :success))
  )
  
;;;-------------------------------------------------------------- (DOC) SHOW-TOPICS

(defdocstate
  _doc-show-topics
  (:label "Show topics")
  (:explanation "Doctor is selecting a list of topics to show patient.")
  (:select-topics 
     ("Palace" "Nijo" "Kiyomizu")
     :question
     "Palace, Nijo Castle, Kiyomizu Temple: have you ever been to ~
		   any place listed here? If so, select it."
     :failure _doc-no-visit)
  )
  
#|
(defdocstate
  _doc-show-topics
  (:label "Show topics")
  (:explanation "Doctor is selecting a list of topics to show patient.")
  ;; the post-topics skill posts a list of topics and asks a question
  (:send-message :to :patient :self *current-agent* 
                 :action :post-topics
                 :args `(((:data ("palace" "Nijo castle" "Kiyomizu")
				                 "Have you ever been to any place listed here? If so, select it.")
                          )))
  ;; post the text
  (:execute-preconditions
	(send moss::conversation 
          '=display-text
		  "Palace, Nijo Castle, Kiyomizu Temple: have you ever been to ~
		   any place listed here? If so, select it."
		:header "DOCTOR> "
		:color cg::red))
  ;; on selection of a topic, we get the answer somewhere
  (:transitions
    (:patterns
     (("Palace")) :sub-dialog _palace-conversation)
	(:patterns
	 (("Nijo")) :sub-dialog _nijo-conversation)
	(:patterns
	 (("Kiyomizu")) :sub-dialog _kiyomizu-conversation)
    (:otherwise :target _doc-no-visit))
  )
|#

;;;================================================================================
;;;
;;;                            PALACE CONVERSATION
;;;
;;;================================================================================

(defsubdialog 
  _palace-conversation 
  (:label "Palace conversation")
  (:explanation "We talk about the palace.")
  (:states _pal-entry-state  ; required by defstate
           _pal-do-not-know
		   _pal-show-topics
		   )
  )
  
;;;------------------------------------------------------------- (PAL) ENTRY-STATE

(defdocstate
  _pal-entry-state
  (:label "Entering palace topic")
  (:entry-state _palace-conversation )
  (:explanation "Topic is Palace.")
  (:ask-patient "Palace is a nice place to visit don't you think so?")
  (:transitions
    (:always :target _pal-show-topics))
  ) 
  
;;;-------------------------------------------------------------- (PAL) DO-NOT-KNOW

(defdocstate
  _pal-do-not-know
  (label "Patient does not know topics")
  (explanation "Patient did not answer or answered negatively.")
  (:tell-patient "Ah, maybe we can tackle these topics later on...")
  (:transitions
     (:always :success))
  )
;;;------------------------------------------------------------- (PAL) SHOW-TOPICS

(defdocstate
  _pal-show-topics
  (:label "Show palace topics")
  (:explanation "Doctor is selecting a list of topics to show patient.")
  (:select-topics ("Sakura" "Emperor" "Tea ceremony")
     :question "Sakura, Emperor,Tea ceremony: Do you know something about these topics?"
	 :failure _pal-do-not-know)
  )
  
#|  
(defdocstate
  _pal-show-topics
  (:label "Show palace topics")
  (:explanation "Doctor is selecting a list of topics to show patient.")
  ;; the post-topics skill posts a list of topics and asks a question
  (:send-message :to :patient :self *current-agent* 
                 :action :post-topics
                 :args `(((:data ("Sakura" "Emperor" "Tea ceremony")
				              "Do you know something about these topics?")
                          )))
  ;; post the text
  (:execute-preconditions
	(send moss::conversation 
          '=display-text
		  "Sakura, Emperor,Tea ceremony: Do you know something about these topics?"
		:header "DOCTOR> "
		:color cg::red))
  ;; on selection of a topic, we get the answer somewhere
  (:transitions
    (:patterns
     (("Sakura")) :sub-dialog _sakura-conversation)
	(:patterns
	 (("Emperor")) :sub-dialog _emperor-conversation)
	(:patterns
	 (("Tea ceremony")) :sub-dialog _tea-ceremony-conversation)
    (:otherwise :target _pal-do-not-know))
  )
|#  
  
;;;================================================================================
;;;
;;;                       PRINT HELP CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to help the master by giving general information:
;;; what tasks are available.

(defsubdialog 
  _print-help-conversation 
  (:label "Help conversation")
  (:explanation "Help was asked.")
  (:states _ph-entry-state  ; required by defstate
           _ph-print-global-help)
  )

;;;-------------------------------------------------------------- (PH) ENTRY-STATE

(defstate 
  _ph-entry-state
  (:label "Explain dialog entry")
  (:entry-state _print-help-conversation )
  (:explanation "Master is asking for help.")
  (:transitions
   ;; make sure we have a question
   (:patterns (((?* ?x) "au" "secours" (?* ?y))
               ("sos")
               ("help")
               ("?")
               ((?* ?x) "Je" "suis" (?* ?y) "perdu" (?* ?z))
               ((?* ?x) "Je" "suis" (?* ?y) "perdue" (?* ?z))
               ("quels" (?* ?x) "vous" "proposez" (?* ?y))
               ((?* ?x) "services" (?* ?y) "vous" "offrez" (?* ?z))
               ("que" "pouvez-vous" "faire" (?* ?y))
               ("quels" "services" (?* ?y))
               )
              :target _ph-print-global-help)
   (:otherwise :failure)
   ))

;;;-------------------------------------------------------- (PH) PRINT-GLOBAL-HELP

(defstate
  _ph-print-global-help
  (:label "Print general help")
  (:explanation "No specific subject was included.")
  (:execute
     (let
         ((obj-id (car (send '>-global-help '=get 'is-title-of)))
          (*language* :fr)
          (task-list (access '(task)))
          (conversation (omas::conversation DOCTOR::PA_DOCTOR))
          )
         (when obj-id (send obj-id '=get-documentation)
               (send conversation '=display-text *answer*))
         (send conversation '=display-text
               "~2%Je peux faire les choses suivantes :")
         (dolist (task task-list)
           (send task '=get-documentation :lead "   - ")   
           (send conversation '=display-text *answer*)
           )
         (send conversation '=display-text "~2%")
         ))
  (:transitions
   (:always :success)))

;;;================================================================================
;;;
;;;                       SET FONT SIZE CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to modify the size of the dialog font, usually to
;;; increase it or to return to the default value.

(defsubdialog 
  _set-font-size-conversation
  (:label "Font size conversation")
  (:explanation 
   "The data is searched for the presence of words like increase, ~
    bigger, smaller, defaults, reset,...")
  (:states _sfs-entry-state  ; required by defstate
           _sfs-dont-understand
           _sfs-try-again
           _sfs-sorry)
  )

;;;--------------------------------------------------- (SFS) FONT-SIZE-ENTRY-STATE

(defstate 
  _sfs-entry-state
  (:entry-state _set-font-size-conversation)
  (:label "Font size dialog entry")
  (:explanation "The data is searched for the presence of words like increase, ~
                 bigger, defaults, reset,...")
  (:transitions
   (:patterns (((?* ?x) "augmenter" (?* ?y))
               ((?* ?x) "plus" "grand" (?* ?y))
               ((?* ?x) "trop" "petit" (?* ?y))
               ((?* ?x) "plus" "gros" (?* ?y)))
              :exec (omas::assistant-use-larger-font *current-agent*) :success)
   (:patterns (((?* ?x) "diminuer" (?* ?y))
               ((?* ?x) "plus" "petit" (?* ?y))
               ((?* ?x) "moins" "grand" (?* ?y))
               ((?* ?x) "moins" "gros" (?* ?y)))
              :exec (omas::assistant-reset-font *current-agent*) :success)
   (:patterns (((?* ?x) "utiliser" (?* ?y) "défaut" (?* ?z))
               ((?* ?x) "reset" (?* ?y))
               ((?* ?x) "défaut" (?* ?y)))
              :exec (omas::assistant-reset-font *current-agent*) :success)
   (:otherwise :target _sfs-dont-understand)
   ))

;;;--------------------------------------------------------- (SFS) DONT-UNDERSTAND

(defstate 
  _sfs-dont-understand
  (:label "Did not understand. Ask master.")
  (:explanation "Assistant is asking master for font-size clarification.")
  (:question "~%- Vous voulez que j'écrive plus gros, plus petit, ou que ~
              j'utilise la taille par défaut ?")
  (:transitions
   (:always :target _sfs-try-again)))

;;;------------------------------------------------------------------- (SFS) SORRY

(defstate 
  _sfs-sorry
  (:label "Font dialog failure.")
  (:explanation "We cannot make things out from the data.")
  (:reset)
  (:answer "~%- Désolé, je ne vois pas ce que vous voulez faire avec les lettres.~%")
  (:transitions
   (:always :failure))
  )

;;;--------------------------------------------------------------- (SFS) TRY-AGAIN

(defstate 
  _sfs-try-again
  (:label "Font size try again")
  (:explanation "we asked master and try to pick up the answer.")
  (:transitions
   (:patterns (((?* ?x) "augmenter" (?* ?y))
               ((?* ?x) "plus" "grand" (?* ?y))
               ((?* ?x) "plus" "gros" (?* ?y)))
              :exec (omas::assistant-use-larger-font *current-agent*) :success)
   (:patterns (((?* ?x) "diminuer" (?* ?y))
               ((?* ?x) "plus" "petit" (?* ?y))
               ((?* ?x) "moins" "grand" (?* ?y))
               ((?* ?x) "moins" "gros" (?* ?y)))
              :exec (omas::assistant-reset-font *current-agent*) :success)
   (:patterns (((?* ?x) "utiliser" (?* ?y) "défaut" (?* ?z))
               ((?* ?x) "reset" (?* ?y))
               ((?* ?x) "défaut" (?* ?y))
               ((?* ?x) "defaut" (?* ?y)))
              :exec (omas::assistant-reset-font *current-agent*) :success)
   (:otherwise :target _sfs-sorry)
   )
  )

;;;================================================================================
;;;
;;;                       TRACE CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to activate or deactivate tracing of the dialog
;;; by printing the states and transitions into the Lisp console

(defsubdialog 
  _trace-conversation
  (:label "Trace conversation")
  (:explanation 
   "Command to toggle the dialog trace.")
  (:states _tr-entry-state)
  )

;;;-------------------------------------------------------- (TR) TRACE-ENTRY-STATE

(defstate 
  _tr-entry-state
  (:label "Trace dialog entry")
  (:entry-state _trace-conversation)
  (:explanation "We search the input for the trace command.")
  
  (:transitions
   ;; system commands
   (:patterns (((?* ?x) "afficher" (?* ?y) "transitions" (?* ?z))
               ((?* ?x) "tracer" (?* ?y) "transitions" (?* ?z))
               ((?* ?x) "imprimer" (?* ?y) "transitions" (?* ?z)))
              :exec (setq moss::*transition-verbose* t
                          moss::*traced-agent* *current-agent*) :success)
   (:patterns (((?* ?x) "pas" "de" "trace" (?* ?y))
               ((?* ?x) "supprimer" "la" "trace" (?* ?y))
               ((?* ?x) "arrêter" "la" "trace" (?* ?y))
               ((?* ?x) "stop" "trace" (?* ?y)))
              :exec (setq moss::*verbose* nil
                          moss::*transition-verbose* nil
                          moss::*traced-agent* nil) :success)
   (:patterns (((?* ?x) "tracer" (?* ?y))
               ((?* ?x) "bavard" (?* ?y)))
              :exec (setq moss::*verbose* t
                          moss::*traced-agent* *current-agent*) :success)
   (:patterns (((?* ?x) "supprimer" (?* ?y) "transitions" (?* ?z))
               ((?* ?x) "stopper" (?* ?y) "transitions" (?* ?z))
               ((?* ?x) "ne" "plus" "montrer" (?* ?y) "transitions" (?* ?z)))
              :exec (setq moss::*transition-verbose* nil
                          moss::*verbose* nil) :success)
   (:otherwise :failure)
   ))

;;;================================================================================
;;;
;;;                      WHAT-IS CONVERSATION
;;;
;;;================================================================================

;;; this conversation is intended to obtain and print information about something
;;; corresponding to unformatted data. When the assistant has not a local answer
;;; it sends a broadcast to the staff agents, collecting their answers.

(defsubdialog 
  _what-is-conversation
  (:label "Get info dialog")
  (:explanation "Master is trying to info about some data.")
  (:states _what-is-conversation  ; required by defstate
           _wai-entry-state
           _wai-ask-staff
           _wai-get-info
           _wai-sorry)
  )

;;;------------------------------------------------------------- (WAI) ENTRY-STATE

(defstate 
  _wai-entry-state
  (:label "What is conversation")
  (:entry-state _what-is-conversation)
  (:explanation "Assistant is trying to get object info from its knowledge base.")
  (:transitions
   ;;keep only relevant data
   (:patterns (((?* ?x) "qu" "est" "ce" "que" (?* ?y) "?")
               ((?* ?y) "c" "est" "quoi" "?" (?* ?x))
               ((?* ?x) "c" "est" "quoi" (?* ?y) "?")
               ((?* ?y))  ; when entering from print-concept-documentation
               )
              :keep ?y
              :target _wai-get-info)
   (:otherwise :target _wai-sorry)
   ))

;;;--------------------------------------------------------------- (WAI) ASK-STAFF
;;; changing strategy to :collect-answers
;;; rather than sending a broadcast right away, maybe we should
;;; 1. look into local KB
;;; 2. look into our agenda (sa-address)
;;; 3. ask BIBLIO
;;; This would avoid strange answers resulting from misinterpretation of the
;;; query.

(defstate 
  _wai-ask-staff
  (:label "Ask staff.")
  (:explanation "Ask the staff for help.")
  (:send-message :to :all :self *current-agent* :action :what-is :timeout 2
                 :args `(((:data . ,(read-fact moss::conversation :input))
                          (:language . :en)
                          ))
                 :strategy :collect-answers
                 )
  (:transitions 
   (:always :target _wai-process-staff-answer)))

;;;---------------------------------------------------------------- (WAI) GET-INFO

(defstate 
  _wai-get-info
  (:label "Looking into local KB info.")
  (:explanation "Look into own KB to get info.")
  (:answer-analysis))

(defownmethod =answer-analysis _wai-get-info (conversation input)
  "we got an answer from some staff or else the master got impatient.
Arguments:
   agent: assistant."
  (declare (special *empty-words*))
  (let* (entry-list)
    (catch :return
      ;; clean input of parasitic words
      (setq input (mapcan #'(lambda (xx) 
                              (if (member xx *empty-words* :test #'string-equal)
                                nil
                                (list xx)))
                          input))
      (moss::vformat "DOCTOR: _wai-get-info /+++ cleaned input:~& ~S" input)
      ;; if nothing left, then ask staff
      (unless input
        (throw :return `(:transition ,_wai-ask-staff)))
      ;; check whether the resulting words correspond to entry points
      (setq entry-list (moss::find-best-entries input))
      (moss::vformat "DOCTOR: _wai-get-info /entry list:~&    ~S" entry-list)
      ;; filter the person objects
      ;; if nothing left go ask staff
      (unless entry-list 
        (throw :return `(:transition ,_wai-ask-staff)))
      ;; if more than one person ask for help (not yet)
      ;; get documentation or CV
      ;; print it
      (dolist (item entry-list)
        (send item '=get-documentation :lead "   - ")   
        (send conversation '=display-text *answer*)
        )
      ;; transfer to :success    
      (list :success) 
      )
    ))

;;;------------------------------------------------------ (WAI) PROCESS-STAFF-ANSWER

(defstate 
  _wai-process-staff-answer
  (:label "Process staff answer.")
  (:explanation "Process anwers from the staff agents.")
  (:answer-analysis)
  )

(defownmethod =answer-analysis _wai-process-staff-answer (conversation input)
  "we got an answer from some staff or else the master got impatient.
Arguments:
   agent: assistant."
  (declare (ignore input))
  (let* ((agent (car (has-agent conversation)))
         (answer-list (omas::answer agent))
         (*language* :en)
         )
    ;(format *debug-io* "~&+++ =resume; HAS-TO-DO: ~S" (has-to-do conversation))
    ;; clean up HAS-TO-DO (contains "answer-there")
    ;(setf (has-to-do conversation) nil)
    ;; answe? contains either the word "failure" or a list of lists of strings
    ;; (("Jean-Paul Barthes is a professor at UTC ..."
    ;;  "Dominique Barthes-Biesel is a professor at UTC ...")
    ;;  ("Jean-Paul Barthes has NN publications on:  ...")
    ;;  ...)
    ;; 
    (moss::vformat "~&>>> WAIdialog /Answer to the send-request:~% ~S" answer-list)
    ;; failure if :failure is returned or :error on timeout
    (cond
     ((or (equal "failure" answer-list) (eql :error answer-list))
      `(:transition ,_wai-sorry))
     ((null answer-list)
      (send conversation '=display-text "Je ne sais pas...")
      (list :success))
     (t 
      (moss::vformat " _wai-ask-staff /answer-list: ~& ~S" answer-list)
      (send conversation '=display-text "~%C'est:")
      ;; each item of the answer list is like 
      ;;    ("personne" ("nom" "...")("prénom" "..."))
      (dolist (answer answer-list)
        ;; answer could be "failure" rather than a list of strings
        (unless (and (stringp answer) (string-equal answer "failure"))
          (dolist (item answer)
            (send conversation '=display-text 
                  (concatenate 'string "~%" (car item)))
            (send conversation '=display-text
                  (format nil ": ~{~%     ~{~A~^, ~}~}" (cdr item))))))
      (list :success)))
    ))

;;;------------------------------------------------------------------- (WAI) SORRY

(defstate 
  _wai-sorry
  (:label "Failure to get info.")
  (:explanation "We could not find anything.")
  (:reset)
  (:text "Sorry, I could not find information for the data.")
  (:transitions 
   (:failure))
  )
;;;================================================================================
;;;
;;;                  CONVERSATIONS AND SUB-CONVERSATIONS
;;;
;;;================================================================================

;;; we declare the top-level conversations
;;; the entry point of the dialog is _MAIN-CONVERSATION

(eval-when (:load-toplevel :compile-toplevel :execute)
  (proclaim '(special 
              _MAIN-CONVERSATION
              _PROCESS-CONVERSATION
              )))

;;;================================================================================
;;;================================================================================
;;;
;;;                       MAIN CONVERSATION (CONTROL LOOP)
;;;
;;;================================================================================
;;;================================================================================

;;; documentation is in English but the main dialog is in English, French, Brasilian, 
;;; or Japanese, according to the *language* variable.

;;; create a dialog header to be used for the main conversation

(defindividual 
  MOSS-DIALOG-HEADER
  (HAS-MOSS-LABEL "Main conversation loop")
  (HAS-MOSS-EXPLANATION "This is the main conversation loop.")
  (:var _main-conversation))

;; declare that conversation, replacing default conversation in the agent structure
(setf (omas::dialog-header PA_DOCTOR) _main-conversation)

;;;===== states are declared as global variables
;;; the set of states can be considered as a plan to be executed for conducting the
;;; conversation with the master

;;;--------------------------------------------------------------------------------
(eval-when (:compile-toplevel :load-toplevel :execute)
  (proclaim '(special 
              _mc-entry-state
              _mc-get-input
              _mc-more?
              _mc-process
              _mc-sleep)))
;;;--------------------------------------------------------------------------------

;;;--------------------------------------------------------------- (MC) ENTRY-STATE
;;; ENTRY STATE
;(defun ff () (start-conversation _main-conversation :first-time t))
;(defun gg () (start-conversation _main-conversation))

;;; define an entry instance and record it in a global variable.
;;; ****** we should enter a fram on the conversation frame list, to handle
;;; success or failure returns from sub-dialogs. I.e. we should make a transition
;;; on the success or failure state of the main conversation.
;;; entry (_mc-failure _mc-success _mc-entry-state nil) ; no goal 

;;; Action:
;;;   =EXECUTE
;;;   - get ACTION if any from the dialog header
;;;   - reset conversation: HAS-MOSS-FACTS, HAS-MOSS-TASK, HAS-MOSS-TASK-LIST
;;;   - if ACTION IS non NIL, set conversation HAS-GOAL with a pattern obtaines from
;;;        sending ACTION a =create-pattern message
;;;     otherwise, reset conversation HAS-GOAL
;;;   - display welcome message
;;;
;;;   <Master enters text>
;;;
;;;   =RESUME
;;;   - if INPUT contains one of the abort commands, throws to :DIALOG-ERROR
;;;   - makes a transition to _MC-PROCESS

(defstate _mc-entry-state
  (:entry-state _main-conversation)
  (:label "Début du dialogue")
  (:explanation 
   "Initial state when the assistant starts a conversation. Send a welcome message ~
    and wait for data. Also entered on a restart following an abort.")
  (:reset-conversation)
  (:text 
   (:fr "Attention! Ce dialogue est trés limité. Chaque phrase est ~
    indépendante et l'on ne peut utiliser des pronoms référents."
	:en "Warning: this dialog is limited. Each sentence is independent ~
	     and pronoun references cannot be used."
	:br "Atenção! Este diálogo é muito limitado. Cada frase é independente ~
         e não é possível utilizar pronomes relativos."
    :jp "これは試しにやってみました"))
  (:question-no-erase
   ((:fr "Bonjour ! que puis-je faire pour vous ?"
     :en "Hello! What can I do for you?"
	 :br "Bom dia ! o que posso fazer por você"
	 :jp "こんにちは．なにか御用ですか?")
    (:fr "Salut ! Que voudriez-vous savoir ?" 
	 :en "Hi! What would you like to know?"
	 :br "Olá ! O que deseja saber ?" 
	 :jp "なにか知りたいことがありますか?")
	(:fr "Bonjour ! Je vais faire de mon mieux pour répondre à vos ~
          questions. Mais, rappelez-vous que mon QI est faible."
	 :en "Hi! I will try to do my best to answer your question or ~
          execute your orders, but remember that my IQ is limited."
	 :br "Bom dia ! Eu vou fazer o possível para responder às suas perguntas. ~
          Mas, lembre-se que o meu QI é fraco."
	 :jp "もしやって欲しいことがあれば、がんばってやります．"))
	)
  (:transitions 
   (:always :target _mc-process))
  )

;;;----------------------------------------------------------------- (MC) GET-INPUT

(defstate _mc-get-input
  (:label "Get input")
  (:explanation
   "State in which the user inputs his request.")
  (:reset)
  (:question 
    (:fr "Je vous écoute...."
	 :en "I am listening..."
	 :br "Sou todo ouvidos ..."
	 :jp "- 入力してください (_mc-get-input)")
	)
  (:transitions
   (:always :target _mc-process)))

;;;-------------------------------------------------------------------- (MC) MORE?
;;; Action:
;;;   - If answer is "rien", "non" transition to SLEEP
;;;   - if answer is "oui" transition to GET-INPUT
;;;   - anything else sends to PROCESS

(defstate _mc-more?
  (:label "More?")
  (:explanation "Asking the user it he wants to do more interaction.")
  (:reset-conversation)
  (:question-no-erase
   ((:fr "Que puis-je faire d'autre pour vous ?"
     :en "What else can I do for you?"
	 :br "O que posso ainda fazer ?"
	 :jp "なにかもっとすることがありますか?"
    )
    (:fr "Y a-t-il autre chose que je puisse faire pour vous ?"
	 :en "Is there anything else I could do for you?"
	 :br "Há outra coisa que eu possa fazer ?"
	 :jp "OK."
	)
    (:fr "Avez-vous d'autres questions ?"
	 :en "Any other question?"
	 :br "Tem outras questões ?"
	 :jp "なにかもっとすることがありますか?"
	)
    (:fr "OK."
	 :en "OK."
	 :br "OK."
	 :jp "OK."
	 )
   ))
  (:transitions 
   (:starts-with ("rien" "nothing" "nada" "なにも") :target _mc-sleep)
   (:no :target _mc-sleep)
   (:yes :target _mc-get-input)
   (:otherwise :target _mc-process)))

;;;------------------------------------------------------------------- (MC) PROCESS

(defstate _mc-process
  (:label "Process state of main conversation")
  (:explanation
   "Here we call the main process organized as a sub-vconversation.")
  (:transitions
   (:always :sub-dialog _process-conversation
            :success _mc-more?
            :failure _mc-more?)))

;;;--------------------------------------------------------------------- (MC) SLEEP

(defstate _mc-SLEEP
  (:label "Nothing to do")
  (:explanation
   "User said she wanted nothing more.")
  (:reset)
  (:text 
   (:fr "OK. J'attends que vous soyez prêt."
    :en "OK. I wait until you are ready."
	:br "OK. Eu aguardo até que esteja pronto."
	))
  (:question-no-erase 
   (:fr "Réveillez moi en tapant ou en disant quelque chose..."
    :en "Wake me up by typing or saying something..."
	:br "Acorde-me digitando ou dizendo alguma coisa..."
	:jp "お待ちください．"
   ))
  (:transitions
   (:always :target _mc-process)))

;;;================================================================================
;;;
;;;                      MAIN CONVERSATION (EXECUTION PART)
;;;
;;;================================================================================

;;; this conversation is intended to process the input from te user by first
;;; determining the performative, then the list of possible tasks

(defsubdialog 
  _process-conversation 
  (:label "Process conversation")
  (:explanation 
   "Processing steps of the main conversation.")
  (:states _mc-eliza
           _mc-find-performative
           _mc-entry-state
           _mc-failure
           _mc-find-performative
           _mc-find-task
           _mc-select-task
           _mc-task-dialog)
  )

;;;===============================================================================
;;;                            Service functions
;;;===============================================================================

;;;-------------------------------------------------------------------- PRINT-TIME

(defun print-time (conversation &optional (language *language*))
  (multiple-value-bind (second minute hour date month year)
                       (decode-universal-time (get-universal-time))
    (send conversation '=display-text
          (case language
            (:fr
             (format nil 
                     "Il est actuellement ~S:~S:~S et nous sommes aujourd'hui le ~
                      ~S/~S/~S"
                     hour minute second date month year))
			(:br
			 (format nil 
                     "A hora atual é ~S:~S:~S e hoje estamos no ~S/~S/~S"
                     hour minute second date month year))
            (:jp
             (format nil "今は ~S:~S:~S ．今日は ~S/~S/~S~%"
                     hour minute second date month year))
		    (otherwise
             (format nil "Current time is ~S:~S:~S and today is the ~S/~S/~S"
                     hour minute second date month year)))
          )))

;;;===============================================================================
;;;                            Conversation states
;;;===============================================================================

;;;--------------------------------------------------------------------- (MC) ELIZA

(moss::defstate _mc-ELIZA
  (:label "ELIZA")
  (:explanation
   "Whenever MOSS cannot interpret what the user is saying, ELIZA is ~
    called to do some meaningless conversation to keep the user happy. ~
	It then record the master's input and transfers to find performative ~
	state.")
  (:eliza)
  (:transitions
   (:always :target _mc-find-performative))
  )

;;;------------------------------------------------------------------- (MC) FAILURE

(defstate _mc-failure
  (:label "Failure state of main conversation")
  (:explanation
   "Failure state is entered when we return from a sub-dialog with a ~
    :failure tag. We check for more tasks to perform (listed in the ~
	task-list slot of the conversation object). If there are more, we ~
	execute the first one. If there are no more, we return with a failure ~
	tag.")
  (:answer-analysis)
  )

(defownmethod
  =answer-analysis _mc-failure (conversation input)
  (declare (ignore input))
  (let* ((task-list (HAS-MOSS-TASK-LIST conversation))
         task)
    (if task-list
      (progn
        ;; record next task
        (setq task (pop task-list))
        ;; remove it from task-list
        (send conversation '=replace 'HAS-MOSS-TASK-LIST task-list)
        ;; add it to conversation
        (send conversation '=replace 'HAS-MOSS-TASK (list task))
        ;; transfer to sub-dialog
        `(:transition ,_mc-task-dialog))
      ;; when no more tasks we return :failure and the state variable still contains
      ;; :failure. The crawler will return one more level until hitting a non
      ;; failure return containing a specific state (e.g. the one for the main
      ;; conversation)
      (list :failure)
      )))

;;;--------------------------------------------------------- (MC) FIND-PERFORMATIVE
;;; for increased efficiency one can suppress from the code all test referring to
;;; a language different from the one used by the assistant.

(defstate _mc-find-performative
  (:entry-state _process-conversation)
  (:process-state _main-conversation)
  (:label "Find performative")
  (:explanation 
   "Process what the user said trying to determine the type of performative ~
    among :request :assert :command. Put the result if any into the performative ~
    slot of the conversation object.")
  (:transitions 
   ;; time?
   (:patterns (("Quelle" "heure" (?* ?x)))
              :exec (print-time moss::conversation :fr) :success)
   (:patterns (("What" "time" (?* ?x)))
              :exec (print-time moss::conversation :fr) :success)
   (:patterns (("Que" "horas" (?* ?x)))
              :exec (print-time moss::conversation :br) :success)
   (:patterns (((?* ?y) "何" "時" (?* ?x))((?* ?y) "何" "日" (?* ?x)))
              :exec (print-time moss::conversation :jp) :success)   ;; now really select performatives
   (:patterns (("qui" (?* ?x))
               ("quel" "est" (?* ?x))
               ("quelle" "est" (?* ?x))
               ("quels" "sont" (?* ?x))
               ("quelles" "sont" (?* ?x))
               ("qu" "est-ce" "que" (?* ?x))
               ("quoi" (?* ?x))
               ("quand" (?* ?x))
               ("où" (?* ?x))
               ("pourquoi" (?* ?x))
               ("qui" (?* ?x))
               ("combien" (?* ?x))
               ("comment" (?* ?x))
               ("est-ce" "que" (?* ?x))
               ("est-il" (?* ?x))
               ("est" "il" (?* ?x))    ; interface vocal ?
               ("est-elle" (?* ?x))
               ("est" "elle" (?* ?x))
               ("sont-ils" (?* ?x))
               ("sont" "ils" (?* ?x))
               ("sont-elles" (?* ?x))
               ("sont" "elles" (?* ?x))
               ("a-t-il" (?* ?x))
               ("a-t-elle" (?* ?x))
               ("ont-ils" (?* ?x))
               ("ont-elles" (?* ?x))
               ((?* ?x) "c" "est" "quoi" (?* ?y))
               ((?* ?x) "c" "est" "qui" (?* ?y))
               ((?* ?x) "?")
               )
              :set-performative (list :request)
              :target _mc-find-task)
   (:patterns (("what" "is" (?* ?x))
               ("what" "are" (?* ?x))
               ("what" (?* ?x))
               ("when" (?* ?x))
               ("where" (?* ?x))
               ("why" (?* ?x))
               ("who" (?* ?x))
               ("whom" (?* ?x))
               ("whose" (?* ?x))
               ("how" "much" (?* ?x))
               ("how" (?* ?x))
               ("is" (?* ?x))
               ("are" (?* ?x))
               ("were" (?* ?x))
               ("was" (?* ?x))
               ("have" (?* ?x))
               ("had" (?* ?x))
               ("do" (?* ?x))
               ("did" (?* ?x))
               ("can" (?* ?x))
               ("could" (?* ?x))
               ("may" (?* ?x))
               ("might" (?* ?x))
               ((?* ?x) "?")
               )
              :set-performative (list :request)
              :target _mc-find-task)
   (:patterns (("que" (?* ?x))
               ("qual" "é" (?* ?x))
               ("quais" "são" (?* ?x))
               ("o que" "é" "que" (?* ?x))
               ("o que" (?* ?x))
               ("quando" (?* ?x))
               ("onde" (?* ?x))
               ("aonde" (?* ?x))
               ("porque" (?* ?x))
               ("quanto" (?* ?x))
               ("como" (?* ?x))
               ("é" "que" (?* ?x))
               ((?* ?x) "é" "o" "que" (?* ?y))
               ((?* ?x) "é" "quem" (?* ?y))
               ((?* ?x) "?")
               )
              :set-performative (list :request)
              :target _mc-find-task)
   (:patterns ( ((?* ?x) "下さ" "い"))
              :set-performative (list :request)
              :target _mc-find-task)
   (:patterns (((?* ?x) "notez" (?* ?y))
               ((?* ?x) "noter" (?* ?y)))
              :set-performative (list :assert)
              :target _mc-find-task)
   (:patterns (((?* ?x) "note" (?* ?y))
               ((?* ?x) "remember" (?* ?y)))
              :set-performative (list :assert)
              :target _mc-find-task)
   (:patterns (((?* ?x) "note" (?* ?y))
               ((?* ?x) "note que" (?* ?y))
               ((?* ?x) "notar" (?* ?y)))
              :set-performative (list :assert)
              :target _mc-find-task)
   (:patterns (((?* ?x) "で" "す" ))
              :set-performative (list :assert)
              :target _mc-find-task)
   (:otherwise 
    :set-performative (list :command)
    :target _mc-find-task))
  )

;;;----------------------------------------------------------------- (MC) FIND-TASK

(defstate _mc-find-task
  (:label "Find task")
  (:explanation 
   "Process the input to determine the task to be undertaken. Combines the ~
    words from the sentence to see if they are entry points for the index ~
	property of any task. Collect all tasks for which there is an index in ~
	the sentence.")
  (:answer-analysis)
  )

(defownmethod
  =answer-analysis _mc-find-task (conversation input)
  "Using input to find tasks If none failure, if one, OK, if more, must ~
   ask user to select one. Checks for an entry point for an index of a task.
Arguments:
   conversation: current conversation
   input: list containing the input as a list of words"
  (let* ((performative (read-fact conversation :PERFORMATIVE))
         task-list)
    (moss::vformat "_mc-find-task /performative: ~S, package: ~S, input:~%  ~S" 
	                               performative *package* input)
    ;; try to find a task from the input text
    (setq task-list (moss::find-objects 
                     '(TASK 
					    (HAS-INDEX-PATTERN 
					       (TASK-INDEX (HAS-INDEX :is :?))))
                     input :all-objects t)) 
	(moss::vformat "_mc-find-task /possible tasks:~%  ~S" task-list)
    ;; filter tasks that do not have the right performative
    (setq task-list
          (mapcan #'(lambda (xx) 
                      (if (intersection performative (HAS-PERFORMATIVE xx))
                        (list xx)))
                  task-list))
    (moss::vformat "_mc-find-task /task list after performative check: ~S" task-list)
    (cond 
     ;; if empty, ask ELIZA
     ((null task-list)
      `(:transition ,_mc-ELIZA))
     ;; if one, then OK
     ((null (cdr task-list))
      ;; save results, make the task the conversation task
      (setf (HAS-MOSS-TASK conversation) task-list)
      `(:transition ,_mc-task-dialog))
     (t 
      ;; otherwise must select one from the results
      (setf (HAS-MOSS-TASK-LIST conversation) task-list)
      `(:transition ,_mc-select-task)))
    ))

;;;--------------------------------------------------------------- (MC) SELECT-TASK

(defstate 
  _mc-select-task
  (:label "mc select task")
  (:explanation 
    "we have located more than one task. We rank the tasks by computing ~
     the average weight of the terms in the task index slot. We then ~
     record the list of tasks into the statte-context task-list slot ~
     and activate the highest ranking task.")
  (:answer-analysis) 
  )

(defownmethod =answer-analysis _mc-select-task (conversation input)
  "we have located more than one task. We rank the tasks by computing ~
   the MYCIN combination weight of the terms in the task index slot. We ~
   then record the list of tasks into the conversation task-list slot ~
   and activate the highest ranking task."
  ;(declare (ignore input))
  ;; the list of tasks is in the HAS-MOSS-TASK-LIST slot. Tasks are defined in the application
  ;; package
  (let* ((task-list (HAS-MOSS-TASK-LIST conversation))
         patterns weights result pair-list selected-task level word-weight-list)
    (moss::vformat "_select-task /input: ~S~&  task-list: ~S" input task-list)
    ;; first compute a list of patterns (combinations of words) from the input
    (setq patterns (mapcar #'car (moss::generate-access-patterns input)))
    (moss::vformat "_select-task /input: ~S~&  generated patterns:~&~S" 
                   input patterns)
    ;; then, for each task
    (dolist (task task-list)
      (setq level 0)
      ;; get the weight list
      ;(setq weights (HAS-INDEX-WEIGHTS task))
      (setq weights (moss::%get-INDEX-WEIGHTS task))
      (moss::vformat "_select-task /task: ~S~&  weights: ~S" task weights)
      ;; check the patterns according to the weight list
      (setq word-weight-list (moss::%get-relevant-weights weights patterns))  
      (moss::vformat "_select-task /word-weight-list:~&  ~S" word-weight-list)
      ;; combine the weights
      (dolist (item word-weight-list)
        (setq level (+ level (cadr item) (- (* level (cadr item))))))
      (moss::vformat "_select-task /level: ~S" level)
      ;; push the task and weight onto the result list
      (push (list task level) result)
      )
    (moss::vformat "_select-task /result:~&~S" result)
    ;; order the list
    (setq pair-list (sort result #'> :key #'cadr)) 
    (moss::vformat "_select-task /pair-list:~&  ~S" pair-list)
    ;; keep the first task whatever its score
    (setq selected-task (caar pair-list))
    ;; remove the task that have a weight less than task-threshold (default 0.4)
    (setq pair-list
          (remove nil 
                  (mapcar 
                   #'(lambda (xx) 
                       (if (>= (cadr xx) (omas::task-threshold omas::*omas*)) xx))
                   pair-list)))
    ;; if task-list is empty then return th? first saved task
    ;; this may not be a good policy if the score is too low
    (if (null pair-list)
      (progn
        ;; reset the task-list slot of the conversation object
        (setf (HAS-MOSS-TASK-LIST conversation) nil)
        ;; put the saved task into the task slot
        (setf (HAS-MOSS-TASK conversation) (list selected-task))
        ;; go to task-dialog
        `(:transition ,_mc-task-dialog)
        )
      (progn
        ;; remove the weights
        (setq task-list (mapcar #'car pair-list))
        ;; select the first task of the list
        (setq selected-task (pop task-list))
        (moss::vformat "_select-task /selected task: ~S" selected-task)
        ;; save the popped list in the task-list slot of the conversation object
        (setf (HAS-MOSS-TASK-LIST conversation) task-list)
        (setf (HAS-MOSS-TASK conversation) (list selected-task))
        ;; go to task-dialog
        `(:transition ,_mc-task-dialog)))
    ))

;;;--------------------------------------------------------------- (MC) TASK-DIALOG

(defstate _mc-task-dialog
  (:label "Task dialog")
  (:explanation 
   "We found a task to execute (in the GOAL slot of the conversation). We ~
    activate the dialog associated with this task.")
  ;; we launch the task dialog as a sub-conversation
  ;; the task contains the name of a sub-conversation header, e.g. _get-tel-nb
  (:answer-analysis)
  )

;;;********** must check for possible problems
(defownmethod
  =answer-analysis _mc-task-dialog (conversation input)
  "We prepare the set up to launch the task dialog.
Arguments:
   conversation: current conversation"
  (declare (ignore input))
  (let* ((task-id (car (send conversation '=get 'HAS-MOSS-TASK))))
    (moss::vformat "_mc-task-dialog /task-id: ~S dialog: ~S" 
                   task-id (HAS-DIALOG task-id))
    ;; we launch the task dialog as a sub-conversation
    ;; the task contains the name of a sub-conversation header, e.g. _get-tel-nb
    `(:sub-dialog ,(car (HAS-DIALOG task-id)) :failure ,_mc-failure)
    ))

;;;================================================================================
;;;================================================================================
;;;
;;;                             ELIZA CONVERSATION
;;;
;;;================================================================================
;;;================================================================================

;;; this conversation is intended to do small talk when no task has been found to
;;; correspond to the master input.

(defParameter *eliza-rules*
  (case *language*
    ;;=== French
    (:fr
  '((("hello")
     "Bonjour ! Je m'appelle DOCTOR."
     "Hello there?")
    (("Bom" "dia")
     "Desculpe ! Não falo Portugues."
     "Bom dia. Por favor fale em frances.")
    (("Salut" (?* ?x))
     "Salut ! Je m'appelle DOCTOR."
     "Salut ?")
    (("bonjour")
     "Bonjour ! Je m'appelle DOCTOR.")
    (("bonjour" (?* ?x))
     "Bonjour, Boss !")
    (("hi")
     "Bonjour ! Je m'appelle DOCTOR. Mon anglais n'est pas bon.")     
    (("comment" "allez-vous" (?* ?x))
     "Très bien merci."
     "Vraiment, ma santé vous intéresse ? J'en suis très flatté.")
    (("qui"  (?* ?x) "Barthès")
     "Jean-Paul Barthès est professeur d'intelligence artificielle à ~
      l'Université de Technologie de Compiègne.")
    (("qui"  (?* ?x) "Barthes")
     "Jean-Paul Barthès est professeur d'intelligence artificielle à ~
      l'Université de Technologie de Compiègne.
      Notez toutefois que Barthès s'écrit avec un è.")
    (("laissez" "tomber")
     "OK. Pas de problème.")
    (("merci" (?* ?y))
     "Avec plaisir.")
    (((?* ?x) "machine" (?* ?y))
     "Est-ce que les machines vous dérangent ?"
     "Que pensez-vous des machines ?"
     "Pourquoi mentionnez-vous les machines ?"
     "Les machines ont-elles un rapport avec votre problème ?")
    (((?* ?x) "stupide" (?* ?y))
     "S'il vous plaît, restez correct !"
     "Je suis peiné par votre remarque.")
    (((?* ?x) "ne" "marche" "pas" (?* ?y))
     "Bien sûr que ?x marche !"
     "Pourquoi dites-vous que ?x ne marche pas ?")
    (("à" "l" "aide")
     "Vous voulez de l'aide à quel sujet?")
    (("help")
     "Vous voulez de l'aide à quel sujet?")
    (((?* ?x) "désolé" (?* ?y))
     "S'il vous plaît, ne vous excusez pas."
     "Des excuses ne sont pas nécessaires"
     "Que ressentez-vous quand vous vous excusez ?")
    (((?* ?x) "Je" "me" "souviens" (?* ?y))
     "Vous souvenez-vous souvent ?y ?"
     "Vous souvenir ?y vous rappelle-t-il autre chose ?"
     "De quoi d'autre vous souvenez-vous ?"
     "Pourquoi vous souvenez-vous ?y maintenant ?"
     "Qu'est-ce qui dans la situation présente vous fait vous souvenir ?y ?"
     "Quel est le lien entre moi et ?y ?")
    (((?* ?x) "vous" "rappelez-vous" (?* ?y))
     "Pensez-vous que j'aurais oublié ?y ?"
     "Pourquoi pensez-vous que je devrais me rappeler ?y maintenant ?"
     "Qu'y a-t-il à propos de ?y ?"
     "Vous avez mentionné ?y ?")
    (((?* ?x) "si" (?* ?y))
     "Pensez-vous que ?y soit probable ?"
     "Souhaitez-vous ?y ?"
     "Que pensez-vous de ?y ?"
     "Vraiment -- si ?y ?")
    (("que" (?* ?x) "faire" "pour" "moi")
     "Tant de choses..."
     "A ce moment précis, je ne vois pas."
     "Qu'est-ce qui vous ferait plaisir ?")
    (((?* ?x) "J" "ai" "rêvé" (?* ?y))
     "Vraiment -- ?y"
     "Avez-vous déjà rêvé ?y tout eveillé ?"
     "Avez-vous déjà ?y auparavant ?")
    (((?* ?x) "rêve" (?* ?y))
     "Qu'est-ce que ce rêve vous suggère ?"
     "Rêvez-vous souvent ?"
     "Quelles personnes apparaissent dans vos rêves ?"
     "Ne pênsez-vous pas que les rêves sont liés à votre problème ?")
    (((?* ?x) "ma" "mère" (?* ?y))
     "Qui d'autre dans votre famille ?y ?"
     "Parlez moi de votre famille.")
    (((?* ?x) "mon" "père" (?* ?y))
     "Votre père ?"
     "A-t-il une grande influence sur vous ?"
     "Qu'est-ce qui vous vient à l'esprit quand vous pensez à votre père ?")
    (((?* ?x) "Je" "voudrais" (?* ?y))
     "Que signifierait ?y ?"
     "Pourquoi voudriez-vous ?y ?")
    (((?* ?x) "Je" "suis" "content" (?* ?y))
     "Vous ai-je aidé à être content ?y ?"
     "Qu'est-ce qui vous rend heureux en ce moment ?"
     "Pouvez-vous m'expliquer pourquoi vous êtes tout à coup content ?y ?")
    (((?* ?x) "Je" "déprime" (?* ?y))
     "J'en suis fort désolé."
     "je suis sûr que ce n'est pas drôle d'être déprimé.")
    (((?* ?x) "est" "comme" (?* ?y))
     "Quelle resemblance voyez-vous entre ?x et ?y ?"
     "En quel sens ?x est comme ?y ?"
     "Pensez-vous qu'il y ait vraiment un lien ?"
     )
    (((?* ?x) "pareils" (?* ?y))
     "Comment ça ?"
     "Quelle resemblance y a-t-il ?")
    (((?* ?x) "pareil" (?* ?y))
     "Quel autre lien voyez-vous ?")
    
    (((?* ?x) "J" "étais" (?* ?y))
     "L'étiez-vous vraiment ?"
     "Je me doutais que vous étiez ?y"
     "Pourquoi me dites-vous que vous étiez ?y maintenant ?")
    (((?* ?x) "Je" "suis" (?* ?y))
     "En quel sens êtes-vous ?y ?"
     "Souhaitez vous être ?y ?")
    (((?* ?x) "suis-je" (?* ?y))
     "Croyez-vous être ?y ?"
     "Désirez-vous être ?y ?"
     "Qu'est-ce que cela ferait si vous étiez ?y ?")
    (((?* ?x) "suis" (?* ?y))
     "Pourquoi dites-vous \"SUIS\" ?"
     "Je ne comprends pas cela.")
    (((?* ?x) "êtes-vous" (?* ?y))
     "Pourquoi êtes-vous intéressé par savoir si je suis ?y ou pas ?"
     "Préfèreriez-vous que je ne sois pas ?y ?")
    (((?* ?x) "vous" "êtes" (?* ?y))
     "Qu'est-ce qui vous fait penser que je suis ?y ?")
    (((?* ?x) "parce" "que" (?* ?y))
     "Est-ce la seule raison ?"
     "Quelle autre raison pourrait-il y avoir ?"
     "Est-ce que cette raison peut expliquer autre chose ?")
    (((?* ?x) "étiez-vous" (?* ?y))
     "Peut-être étais-je ?y"
     "Qu'en pensez-vous ?"
     "Et si j'avais été ?y ?")
    (((?* ?x) "je" "ne" "peux" "pas" (?* ?y))
     "Peut-être pouvez-vous ?y maintenant ?"
     "Et si vous pouviez ?y ?")
    (((?* ?x) "Je" "ressents" (?* ?y))
     "Ressentez-vous ?y souvent ?")
    (((?* ?x) "J" "ai" "ressenti" (?* ?y))
     "Qu'avez-vous ressenti d'autre ?")
    (((?* ?x) "pourquoi" "ne" "pas" (?* ?y))
     "Ne devriez-vous pas ?y vous-même ?"
     "Pensez-vous que je pourrais ?y ?"
     "Peut-petre pourrais-je ?y en temps utile")   
    (((?* ?x) "oui" (?* ?y))
     "Vous me semblez très positif."
     "Vous êtes sûr ?"
     "Je comprends.")
    (((?* ?x) "non" (?* ?y))
     "Pourquoi non ?"
     "Vous me semblez très négatif."
     "Est-ce que vous dites \"NON\" simplement pour être négatif ?")
    (((?* ?x) "quelqu" "un" (?* ?y))
     "Pouvez-vous être plus précis ?")
    (((?* ?x) "tout" "le" "monde" (?* ?y))
     "Sûrement pas tout le monde."
     "Pensez-vous à quelqu'un en particulier ?"
     "Qui par exemple ?"
     "Vous pensez à quelqu'un de précis ?")
    (((?* ?x) "toujours" (?* ?y))
     "Pouvez-vous donner un exemple précis ?"
     "Quand ?"
     "What incident are you thinking of ?"
     "Toujours -- vraiment ?")
    (((?* ?x) "peut-être" (?* ?y))
     "Vous n'avez pas l'air tout à fait sûr.")
    (((?* ?x) "est" (?* ?y))
     "Vous pensez qu'il ne pourrait pas être ?y ?"
     "Il est peut-être ?y")
    (((?* ?x) "OK" (?* ?y))
     "Bon, comme vous voudrez.")
    
    (((?* ?x))
     "Je ne vois pas ce que vous voulez exactement."
     "Je ne suis pas sûr de comprendre ce que vous voulez."
     "Pouvez-vous reformuler votre question s'il vous plaît ?"
     "Désolé, je n'ai pas compris."
     "Aïe ! Je n'ai pas compris."
     "Pouvez-vous dire cela différemment ?"
     "Pardon ?"
     "Quoi ?"
     "Comment ?"
     "J'ai entendu ?x , mais je ne comprends pas."
     )))
	 
   ;;=== now Brasilian
   (:br
     '((("Ola")
     "Bom dia! Eu me chamo DOCTOR."
     "Ola!")
    (("Bom" "dia")
     "Bom dia ! Tudo bem ?"
     "Bom dia. Encantado..."
     "Bom dia, Chefe !")
    (("Salve" (?* ?x))
     "Olá ! Eu me chamo DOCTOR e falo português."
     "Salve ?")
    (("bom dia " (?* ?x))
     "Bom dia, Chefe !")
    (("bonjour")
     "Bom dia ! Eu me chamo DOCTOR. Não falo frances. Você não fala português?")
    (("hi")
     "Bom dia ! Eu me chamo Zé. Meu inglês não é muito bom.")
    (("como" "vai" (?* ?x))
     "Bem, obrigado!"
     "Sério, minha saúde lhe interessa ? Fico contente!")
    
    (("deixe" "estar")
     "OK. Sem problema.")
    (("obrigado" (?* ?y))
     "Não há de que!")
    (("tudo" (?* ?y))
     "Mesmo ?"
     "Exigente...")
    
    (((?* ?x) "máquina" (?* ?y))
     "As máquina o incomodam ?"
     "O que você pensa das máquinas ?"
     "Por que você menciona as máquinas ?"
     "As máquinas têm relação com o seu problema ?")
    
    (((?* ?x) "estupido" (?* ?y))
     "Por favor, olha a educação !"
     "Lamento a sua observação.")
    
    (((?* ?x) "não" "funciona" (?* ?y))
     "É claro que ?x funciona !"
     "Pour que você diz que ?x não funciona ?")
    
    (("socorro")
     "Como eu poderia ajudá-lo ?")
    
    (("help")
     "Voucê quer ajuda sobre o que ?")
    
    (("SOS")
     "O que posso fazer para ajudar ?")
    
    (((?* ?x) "lamento" (?* ?y))
     "Por favor, não se lamente."
     "Não é necessário")
    
    (((?* ?x) "Eu" "me" "lembro" (?* ?y))
     "Você lembra-se com frequência ?y ?"
     "Do que mais você se lembra ?"
     "Por que você se lembra de ?y agora ?")
    
    (((?* ?x) "você" "lembra-se" (?* ?y))
     "Você pensa que eu esqueceria ?y ?"
     "Por que pensa que eu deveria me lembrar ?y justo agora ?"
     "O que há em relação a ?y ?"
     "Você mencionou ?y ?")
    
    (("o" "que" (?* ?x) "fazer" "por" "mim")
     "Tantas coisas..."
     "Neste momento preciso, não vejo o que."
     "O que poderia lhe agradar ?")
    
    (((?* ?x) "minha" "mãe" (?* ?y))
     "Quem mais na sua familia ?y ?"
     "Fale mais da sua familia.")
    (((?* ?x) "meu" "pai" (?* ?y))
     "Seu pai ?"
     "Ele tem uma grande influência sobre você ?"
     "O que lhe vem ao espirito quando pensa no seu pai ?")
    
    (((?* ?x) "Eu" "gostaria" (?* ?y))
     "O que significaria ?y ?"
     "Por que você gostraia de ?y ?")
    (((?* ?x) "Eu" "estou" "contente" (?* ?y))
     "Eu ajudei a estar contente ?y ?"
     "O que o deixa feliz neste momento ?"
     "Poderia me explicar por que ficou contente de repente ?y ?")
    (((?* ?x) "eu" "deprimido" (?* ?y))
     "Lamento muito !."
     "Estou certo que não é agradável ficar deprimido.")
    (((?* ?x) "é" "como" (?* ?y))
     "Que semelhança você vê entre ?x e ?y ?"
     "Em que sentido ?x é como ?y ?"
     "Você acha mesmo que há uma relação ?")
    
    (((?* ?x) "parecidos" (?* ?y))
     "Como assim ?"
     "Que semelhança existe ?")
    (((?* ?x) "parecido" (?* ?y))
     "Que outra relação você vê ?")
    
    (((?* ?x) "eu" "estava" (?* ?y))
     "Esteve mesmo ?"
     "Eu duvidava que você estivesse ?y"
     "Por que você me diz agora que esteve ?y ?")
    
    (((?* ?x) "eu" "era" (?* ?y))
     "Era mesmo ?"
     "Eu duvidava que você fosse ?y"
     "Por que você me diz agora que era ?y ?")
    
    (((?* ?x) "Eu" "sou" (?* ?y))
     "Em que sentido você é ?y ?"
     "Você deseja ser ?y ?")
    
    (((?* ?x) "porque" (?* ?y))
     "Esta é a única razão ?"
     "Que outra razão poderia haver ?"
     "Esta razão pode explicar outra coisa ?")
    
    (((?* ?x) "eu" "não" "posso" (?* ?y))
     "Talvez pudesse ?y agora ?"
     "E se você pudesse ?y ?")
    
    (((?* ?x) "eu" "senti" (?* ?y))
     "Sentiu algo mais ?")
    (((?* ?x) "porque" "não" (?* ?y))
     "Você não deveria ?y você mesmo ?"
     "Você pensa que eu poderia ?y ?")    
    (((?* ?x) "sim" (?* ?y))
     "Você me parece muito positivo."
     "Tem certeza ?"
     "Eu compreendo.")
    (((?* ?x) "não" (?* ?y))
     "Por que não ?"
     "Você me parece muito negativo.")
    (((?* ?x) "algém" (?* ?y))
     "Poderia ser mais preciso ?")
    (((?* ?x) "todo" "o" "mundo" (?* ?y))
     "Certamente nem todo o mundo!"
     "Pensa em alguém em particular ?"
     "Quem por exemplo ?")
    (((?* ?x) "sempre" (?* ?y))
     "Poderia dar um exemplo preciso ?"
     "Quando ?"
     "Sempre --, mesmo ?")
    (((?* ?x) "talvez" (?* ?y))
     "Você não parece muito seguro...")
    (((?* ?x) "OK" (?* ?y))
     "Bem, como quiser!")
    (((?* ?x) "imbecil" (?* ?y))
     "Oh!"
     "Vou me queixar ao sindicato")
    (((?* ?x) "idiota" (?* ?y))
     "Nem tanto ça..."
     "Não mais que a média."
     "Se você diz...")
    
    (((?* ?x))
     "Não vejo o que você quer exatamente."
     "não estou certo de ter ccompreendido o que você quer."
     "Poderia reformular sua pergunta, por favor ?"
     "Lamento, não entendi."
     "Putz ! Não entendi."
     "Poderia dizer isto de outra forma ?"
     "Perdão ?"
     "O que ?"
     "Como ?"
     "Eu entendi ?x , mas não compreendi."
     )
    ))
	
   ;;=== some Japanese mixed with English
   (:jp
     '((("hello")
     "こんにちは．私は織部です?")
	(("こんにちは")
     "こんにちは．私は織部です?")
	(("もしもし")
     "なにか御用ですか?")
	(("はじめまして")
    	"Enchanté, Je suis DOCTOR."
     	"How do you do, I am DOCTOR.")
	(((?* ?x) "の" "体調" "は" "どうですか")
    	 "?xはとても元気です")
	(((?* ?x) "の" (?* ?y) "は" "どうですか")
    	 "?xの?yは晴れです")
    (("Hi" (?* ?x))
     "なにか御用ですか")
	 
	(((?* ?x))
     "I do not understand exactly what you want. You may type: help."
     "I am not sure I understand what you want. You may type: help."
     "Could you rephrase your question please?"
     "Sorry I did not get that. Could you rephrase it? Or type: help."
     "Oops I do not understand. Could you say that differently?"
     "Sorry?")
	 
	 )
	)

   ;;=== we keep English as a back up language
   (otherwise
    '((("hello")
     "Hello! I am DOCTOR."
     "Hello there?")
    (("Hi" (?* ?x))
     "Hi! Boss."
     "Hi!")
    (("bonjour")
     "Je suis désolé mon français n'est pas encore utilisable. Please use English.")
    (("how" "are" "you" (?* ?x))
     "Very well thank you."
     "Really does my health concern you? I am flattered.")
    (("who"  (?* ?x) "Barthès")
     "Jean-Paul Barthès is a professor of artificial intelligence at the ~
      Université de Technologie de Compiègne in France.")
    (("who"  (?* ?x) "Barthes")
     "Jean-Paul Barthès is a professor of artificial intelligence at the ~
      Université de Technologie de Compiègne in France.
      Note however that Barthès is written with an è.")
    (("forget" "it")
     "OK. No hard feelings.")
    (("thank" "you" (?* ?y))
     "you are welcome."
	 "don't mention it.")
    (("thanks")
     "you are welcome."
	 "don't mention it.")
    (((?* ?x) "computer" (?* ?y))
     "Do computer worry you?"
     "What do you think about machines?"
     "Why do you mention computers?"
     "What do you think machines have to do with your problem?")
    (((?* ?x) "stupid" (?* ?y))
     "Please, behave!"
     "I am pained by your remark.")
    (((?* ?x) "does" "not" "work" (?* ?y))
     "Of course ?x works!"
     "What are  you saying that ?x does not work?")
    (("help")
     "you want help about what?")
    (((?* ?x) "sorry" (?* ?y))
     "Please don't apologize"
     "Apologies are not necessary"
     "What feelings do you have when you apologize?")
    (((?* ?x) "I" "remember" (?* ?y))
     "Do you often think of ?y"
     "Does thinking of ?y bring anything else to mind?"
     "What else do you remember?"
     "Why do you recall ?y right now?"
     "What in the present situation reminds you of ?y ?"
     "What is the connection between me and ?y ?")
    (((?* ?x) "do" "you" "remember" (?* ?y))
     "Did you think I would forget ?y ?"
     "Why do you think I should recall ?y by now ?"
     "What about ?y ?"
     "You mentined ?y ?")
    (((?* ?x) "if" (?* ?y))
     "Do you really think its likely that ?y ?"
     "Do you wish that ?y ?"
     "What do you think about ?y ?"
     "Really -- if ?y ?")
    (("what" (?* ?x) "do" "for" "me")
     "So many things..."
     "Currently I do not know."
     "What could please you?")
    
    (((?* ?x) "I" "dreamt" (?* ?y))
     "Really -- ?y"
     "Have you ever fantasized ?y while you were awake ?"
     "Have you dreamt ?y before ?")
    (((?* ?x) "dream" "about" (?* ?y))
     "How do you feel about ?y in reality ?")
    (((?* ?x) "dream" (?* ?y))
     "What does this dream suggest to you ?"
     "Do you dream often ?"
     "What persons appear in your dreams ?"
     "Don't you believe that dream has to do with your problem ?")
    (((?* ?x) "my" "mother" (?* ?y))
     "Who else in your family ?y ?"
     "Tell me more about your family")
    (((?* ?x) "my" "father" (?* ?y))
     "Your father ?"
     "Does he influence you strongly ?"
     "What else comes to mind when you think of your father ?")
    (((?* ?x) "I" "want" (?* ?y))
     "What would it mean if you got ?y ?"
     "Why do you want ?y ?"
     "Suppose you got ?y soon ?")
    (((?* ?x) "I" "am" "glad" (?* ?y))
     "How have I helped you to be ?y ?"
     "What makes you happy just now ?"
     "Can you explain why you are suddenly ?y ?")
    (((?* ?x) "I" "am" "sad" (?* ?y))
     "I am sorry to hear you are depressed"
     "I am sure it's not pleasant to be sad")
    (((?* ?x) "are" "like" (?* ?y))
     "What resemblance do you see between ?x and ?y ?")
    (((?* ?x) "is" "like" (?* ?y))
     "In what way is it that ?x is like ?y ?"
     "What resemblance do you see ?"
     "Could there really be some connection ?"
     (How ?))
    (((?* ?x) "alike" (?* ?y))
     "In what way ?"
     "What similarities are there ?")
    (((?* ?x) "same" (?* ?y))
     "What other connections do you see ?")
    
    (((?* ?x) "I" "was" (?* ?y))
     "Were you really ?"
     "Perhaps I already knew you were ?y"
     "Why do you tell me you were ?y by now ?")
    (((?* ?x) "was" "I" (?* ?y))
     "What if you were ?y ?"
     "Do you think you were ?y ?"
     "What would it meant if you were ?y ?")
    (((?* ?x) "I" "am" (?* ?y))
     "In what way are you ?y ?"
     "Do you want to be ?y ?")
    (((?* ?x) "am" "I" (?* ?y))
     "Do you believe you are ?y ?"
     "Would you want to be ?y ?"
     "You wish I would tell you you are ?y"
     "What would it mean if you were ?y ?")
    (((?* ?x) "am" (?* ?y))
     "Why do you say \"AM?\""
     "I don't understand that")
    (((?* ?x) "are" "you" (?* ?y))
     "Why are you interested in whether I am ?y or not ?"
     "Would you prefer if I weren't ?y ?"
     "Perhaps I am ?y in your fantasies")
    (((?* ?x) "you" "are" (?* ?y))
     "What makes you think I am ?y ?")
    (((?* ?x) "because" (?* ?y))
     "Is that the real reason ?"
     "What other reasons might there be ?"
     "Does that reason seem to explain anything else ?")
    (((?* ?x) "were" "you" (?* ?y))
     "Perhaps I was ?y"
     "What do you think ?"
     "What if I had been ?y ?")
    (((?* ?x) "I" "can't" (?* ?y))
     "Maybe you could ?y now"
     "What if you could ?y ?")
    (((?* ?x) "I" "feel" (?* ?y))
     "Do you often feel ?y ?")
    (((?* ?x) "I" "felt" (?* ?y))
     "What other feelings do you have ?")
    (((?* ?x) "I" (?* ?y) "you" (?* ?z))
     "Perhaps in your fantasy we ?y each other")
    (((?* ?x) "why" "don't" "you" (?* ?y))
     "Should you ?y yourself ?"
     "Do you believe I don't ?y ?"
     "Perhaps I will ?y in good time")
    (((?* ?x) "yes" (?* ?y))
     "You seem quite positive."
     "You are sure ?"
     "I understand.")
    (((?* ?x) "no" (?* ?y))
     "Why not ?"
     "You are being a bit negative"
     "Are you saying \"NO\" just to be negative?")
    
    (((?* ?x) "someone" (?* ?y))
     "Can you be more specific ?")
    (((?* ?x) "everyone" (?* ?y))
     "Surely not everyone."
     "Con you think of anyone in particular ?"
     "Who for example ?"
     "You are thinking of a special person ?")
    (((?* ?x) "always" (?* ?y))
     "Can you think of a specific example ?"
     "When ?"
     "What incident are you thinking of ?"
     "Really -- always ?")
    (((?* ?x) "perhaps" (?* ?y))
     "You do not seem quite certain")
    (((?* ?x) "are" (?* ?y))
     "Did you think there might not be ?y ?"
     "Possibly they are ?y")
    (((?* ?x))
     "I do not understand exactly what you want. You may type: help."
     "I am not sure I understand what you want. You may type: help."
     "Could you rephrase your question please?"
     "Sorry I did not get that. Could you rephrase it? Or type: help."
     "Oops I do not understand. Could you say that differently?"
     "Sorry?"
     )))
	))


(format t "~%;*** MOSS v~A - DOCTOR dialogs loaded ***" moss::*moss-version-number*)

;;;=============================================================================== 

:EOF