﻿;;;-*- Mode: Lisp; Package: "PATIENT" -*-
;;;===============================================================================
;;;11/04/16
;;;             AGENT PATIENT: ONTOLOGY (file PATIENT-ontology.lisp)
;;;             Copyright Barthès@UTC, 2011
;;;
;;;=============================================================================== 

#|
PATIENT is an English speaking assistant agent.
Its ontology is used to process dialogs with its master.

The ontology is defined using the PDM/MOSS formalism. However it is compat-
ible with SOL and this can produce OWL, HTML, JENA or SPARQL outputs.
|#
(in-package :PATIENT)


(defontology 
  (:title "PATIENT ontology")  ; currently ignored
  (:version "1.0")  ; currently ignored
  (:package "PATIENT") ; currently ignored
  (:language :EN)  ; required for a PA
  )

;;;===============================================================================
;;;
;;;                             CONCEPTS
;;;
;;;===============================================================================

;;;-------------------------------------------------------------------------------
;;;                          CONVERSATION TURN
;;;-------------------------------------------------------------------------------

(defconcept "Conversation Turn"
  (:doc "A CONVERSATION TURN contains patterns to analyze the comment and produce ~
         an anwer")
  (:att "patterns")
  (:att "answers")
  )

;;;-------------------------------------------------------------------------------
;;;                               TOPIC
;;;-------------------------------------------------------------------------------

(defconcept "Topic"
  (:doc "A TOPIC is a possible subject of conversation.")
  (:att "label" (:entry))
  (:att "rules")
  (:att "welcome" (:doc "initial question or address to the user"))
  (:rel "conversation turn" (:to "conversation turn"))
  (:att "question" (:doc "question after the choice is posted"))
  (:att "try again" (:doc "text when we want the user to try again"))
  (:rel "sub-topics" (:to "Topic"))
  (:att "failure" (:doc "text printed after a failure"))
  )

;;;===============================================================================
;;;
;;;                               METHODS
;;;
;;;===============================================================================
  
;;;-------------------------------------------------------------- (TOPIC) =summary

(definstmethod =summary TOPIC ()
    (has-label))

;;;===============================================================================
;;;=============================================================================== 
;;;
;;;                                INDIVIDUALS
;;;
;;;=============================================================================== 
;;;===============================================================================

;;;-------------------------------------------------------------------------------
;;;                                  EMPEROR
;;;-------------------------------------------------------------------------------

(defindividual "Topic"
   ("label" "Emperor")
   ("welcome" "Sorry, I cannot continue from there, my knowledge is too limited."
              "Sorry, I am tired. We must stop."
			  "This was a test, could we start all over again?")
   ("rules" "to be done")
   )

;;;-------------------------------------------------------------------------------
;;;                               KIYOMIZU TEMPLE
;;;-------------------------------------------------------------------------------

(defindividual "Topic"
   ("label" "Kiyomizu Temple")
   ("rules" "to be done")
   ("welcome" "Sorry, I really do not know Kiyomizu Temple well enough.")
   )

;;;-------------------------------------------------------------------------------
;;;                                 NIJO CASTLE
;;;-------------------------------------------------------------------------------

(defindividual "Topic"
   ("label" "Nijo Castle")
   ("rules" "to be done")
   ("welcome" "Sorry, Nijo Castle is not my favorite topic.")
   )

;;;-------------------------------------------------------------------------------
;;;                                   PALACE
;;;-------------------------------------------------------------------------------
#|
When entering a new state
   - a question is asked to the patient
   - the patient's answer is ignored, and a transition leads to sub-topics
   - a choice of new topics is offered (state new-topics) and a question is asked at
     the same time, 
   - if the answer is the selection, the corresponding sub-conversation is called
   - if the answer is typed, then we transfer to the try-again state, giving another
     chance, e.g. asking : "Would you like to discuss one of the topics by clicking
	 the selection?"
   - If the answer is a click then we trnasfer to the sub-dialog, otherwise we give
     up, trnasferring to the failure state.
Thus we should generate several states:
   entry-state, post-topics, try-again, failure
Topic data is used as follows:
   - welcome -> welcome
   - sub-topics, question -> post-topics
   - again -> try-again
   - failure -> failure
|#

(defindividual "Topic"
   ("label" "Palace")
   ("rules" "to be done")
   ("welcome" "Palace is a nice place to visit don't you think so?"
              "I like the Palace. Do you?"
			  "The Palace is neat, isn't it?")
   ("sub-topics" "emperor" "sakura" "tea ceremony")
   ("question" "Sakura, Emperor, Tea ceremony: Do you know something about these topics?"
                "Can we talk about Sakura, Emperor, Tea ceremony?")
   ("failure" "Ah, maybe we can tackle these topics later on...")
   )

;;;-------------------------------------------------------------------------------
;;;                                 SAKURA
;;;-------------------------------------------------------------------------------

(defindividual "Topic"
   ("label" "Sakura")
   ("rules" "to be done")
   )
;;;-------------------------------------------------------------------------------
;;;                               TEA CEREMONY
;;;-------------------------------------------------------------------------------

(defindividual "Topic"
   ("label" "Tea Ceremony")
   ("rules" "to be done")
   )

;;;-------------------------------------------------------------------------------
;;;                                 WELCOME
;;;-------------------------------------------------------------------------------

(defindividual "Topic"
   ("label" "Welcome")
   ("rules" "to be done")
   ("welcome" "Now we are going to talk using topics displayed on the screen. OK? ")
   ("question" "Have you ever been to any place listed here? If so please select it.")
   ("sub-topics" "palace" "nijo castle" "kiyomizu temple")
   )

;;;=============================================================================== 

:EOF