/**
 * Set of functions for dialog management
 * Author: Marcio Fuckner
 *
 * Revision history:
 * 02/11/2013 - MFK - Interface Redesign
 */

// global variables 
var labelFunctions = {}, // label functions used for graph visualization 
    count = 0;

/**
 * Checks if the response is in JSON format
 * Parameters:
 *   response: the string containing the response
 * Returns:
 *   a boolean value.
 */
function isJSON(response) {
   return response.trim().substring(0,1) == '[' || response.trim().substring(0,1) == '{';
}

/**
 * Adds a new empty div for the history composition
 * Returns:
 *   the new div
 */
function addHistoryDiv() {
  var historyDiv = document.getElementById('history');
  var innerDiv = document.createElement('div');
  innerDiv.id = "message" + (++ count); 
  innerDiv.className = "message";
  historyDiv.appendChild(innerDiv);
  return innerDiv;
}

/**
 * Return the current time
 * Returns:
 *   a string containing the current time in the format hh:mm
 */
function currentTime() {
  var date = new Date(); 
  return date.getHours() + ":" + date.getMinutes();
}

/**
 * Creates a div with the user message
 * Parameters:
 *   query: the string containing the user message
 * Returns:
 *   none
 */
function addUserMessage(query) {
   var div = addHistoryDiv();
   div.innerHTML = "<label>User (" + currentTime() + ")</label>" + query ;
}

/**
 * Process the PA response
 * Parameters:
 *   response: The string containing the response
 *   query: The string containing the user utterance
 * Returns:
 *   none
 */
function processResponse(response,query) {

  addUserMessage(query);

  var div = addHistoryDiv();

  if (isJSON(response)) {
  
    response = response.substring(0,response.lastIndexOf("}") + 1);
  
    var envelope = JSON.parse(response);
	 
	if(envelope["type"] == null || envelope["type"] != "pa-message-envelope") {
	  alert("Malformed envelope");
	  return;
	}
	 
	var messages = envelope["payload"];
	 
	for(var i=0;i<messages.length;i++) {
	    
	  if(messages[i].type == "pa-graph-message") {
        div.style.height = "400px";
        div.style.overflow = "hidden";
        renderGraph(messages[i].content,labelFunctions,div.id);
	  } else if(messages[i].type == "pa-chart-message") {
		div.style.height = "400px";
		renderChart(messages[i],div.id);
      }
    }	 
  } else {
     div.innerHTML = "<label>Assistant (" + currentTime() + ")</label>" + response;  
  }
  
  document.getElementById("user-input").value = "";
  document.getElementById("user-input").focus();
  div.scrollIntoView(false);
  document.body.style.cursor = "default";
  
  
}

/**
 * Process the user request
 * Parameters:
 *   query: The user utterance
 * Returns:
 *   none
 */
function processRequest(query) {	
	var ajaxObject ;
	if (window.XMLHttpRequest) {
		ajaxObject = new XMLHttpRequest();
		
		var encodedQuery = encodeURIComponent(query);
		var url = '/webqueryrequest?user-input=' + encodedQuery;

		ajaxObject.open("GET",url,true);
		ajaxObject.send();
		
		document.body.style.cursor = "wait";

		ajaxObject.onreadystatechange = function() {
			if (ajaxObject.readyState == 4 && ajaxObject.status == 200) {
			  processResponse(ajaxObject.responseText,query)
			}
		}
   }
}

/**
 * Checks if the phrase could be submitted
 * Parameters:
 *   the onKeyPress event
 * returns:
 *   none
 */
function checkKey(e) {
  var character = String.fromCharCode(e.keyCode);
  if(e.keyCode == 13 || character == '.' || character == '!' || character == '?') {
    document.getElementById("btn").click();
  }
}

/* EOF */